/*
 * Analyzer.java
 *
 * Created on 2005-apr-12
 * 
 */

package analyzer;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Collections;
import java.util.LinkedList;
import java.util.ListIterator;

import rainbowie.Requirement;
import rainbowie.RequirementList;

/**
 * Defines an analyzer for any technique. Each analyzer should extend this
 * class and overide methods as needed.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class Analyzer { 

    protected int canvasWidth, canvasHeight;
    protected double maxValue;
    protected LinkedList pointList;
    protected RequirementList rList;
    protected GraphPoint selectedPoint;
    protected boolean relativeSupportLines;
    
	
    /**
     * Constructs an analyzer
     * 
     * @param rList		   list of Requirements to be analyzed
     * @param canvasWidth  the width of the drawing area
     * @param canvasHeight the height of the drawing area
     */
    public Analyzer(RequirementList rList, int canvasWidth, int canvasHeight) {
		this.rList = rList;
		this.canvasWidth = canvasWidth;
		this.canvasHeight = canvasHeight;
		setMaxValue();
	}
    
    /**
     * Calculates the highest values among the requirements
     */
    private void setMaxValue() {
        maxValue = 0;
        for (int i=0;i<rList.getLength();i++) {
            maxValue = (rList.getReq(i).getPriority1() > maxValue ? 
                    rList.getReq(i).getPriority1() : maxValue);
            maxValue = (rList.getReq(i).getPriority2() > maxValue ? 
                    rList.getReq(i).getPriority2() : maxValue);          
        }
    }

    /**
     * Creates and assigns the positions of all the requirements to be drawn
     * in the GraphWindow.
     * 
     * @return a list containing all the GraphPoints to be drawn
     */
    public LinkedList createPoints() {
        LinkedList oldList = null;
	    if(pointList!=null) {
	        oldList = pointList;
	    }
	    pointList = new LinkedList();
	    for(int i=0;i<rList.getLength();i++) {
	        if(rList.getReq(i).getPriority1()!=-1.0) {
	            GraphPoint gp = new GraphPoint(rList.getReq(i));
	            double y = rList.getReq(i).getPriority1();
	            double x = rList.getReq(i).getPriority2();
	            gp.setY(new Double(canvasHeight-40-y*(canvasHeight-80)/maxValue).intValue());
	            gp.setX(new Double(40+x*(canvasWidth-80)/maxValue).intValue());
	            pointList.add(gp);
	        }
	    }
	    setReleaseNbrs();
	    if(oldList!=null) {
		    for(int i=0;i<oldList.size();i++) {
		        if(((GraphPoint) oldList.get(i)).isHighLighted()) {
		            ((GraphPoint) pointList.get(i)).highlight();
		            selectedPoint = ((GraphPoint) pointList.get(i));
		        }
		    }
		}
		return pointList;
    }
    
    /**
     * Examines how many different releases that are included and assigns
     * each release an index. This is done because we want each release
     * to be drawn in its own way, so we can tell them apart in the graph
     */
    protected void setReleaseNbrs() {
	    LinkedList releases = new LinkedList();
	    for(int i=0;i<pointList.size();i++) {
	        if( !releases.contains(
	                ((GraphPoint) pointList.get(i)).getRequirement().getRelNbr()) && 
	                ((GraphPoint) pointList.get(i)).getRequirement().getRelNbr()
	                .compareTo("")!=0 ) {
	            releases.add(((GraphPoint) pointList.get(i)).getRequirement().getRelNbr());
	        }
	    }
	    Collections.sort(releases);
	    for(int i=0;i<pointList.size();i++) {
	        String currentRel = ((GraphPoint) pointList.get(i)).getRequirement().getRelNbr();
	        int index = releases.indexOf(currentRel);
	        ((GraphPoint) pointList.get(i)).setRelease(index);
	    }
    }
    
    /**
	 * Selects and highlights the point associated with the given Requirement.
	 * 
	 * @param r the requirement to be highlighted
	 */
    public void highlightPoint(Requirement r) {
        if(selectedPoint!=null) {
			selectedPoint.unhighlight();
		}
        selectedPoint = null;
        for(int i=0;i<pointList.size();i++) {
            GraphPoint gp = (GraphPoint) pointList.get(i);
            if(gp.getRequirement().equals(r)) {
                selectedPoint = gp;
				gp.highlight();
            }
        }
    }
    
    /**
	 * Selects, highlights and returns the first Requirement at the given 
	 * position.
	 * 
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @return  the Requirement at (x, y) or null if no requirment was found
	 */
    public Requirement selectPoint(int x, int y) {
        ListIterator lItr = pointList.listIterator();
		if(selectedPoint!=null) {
			selectedPoint.unhighlight();
		}
		selectedPoint = null;
		while (lItr.hasNext()) {
			GraphPoint gp = (GraphPoint) lItr.next();
			if ( gp.selectedBy(x, y) ) {
				selectedPoint = gp;
				gp.highlight();
				return gp.getRequirement();
			}
		}
		return null;
    }
    
    /**
	 * Returns the selected point's Requirement.
	 * 
	 * @return the selected point's Requirement
	 */
    public Requirement getSelectedPointsRequirement() {
        return selectedPoint.getRequirement();
    }
    
    /**
	 * Returns a list with all the points at the given position.
	 * 
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @return  a LinkedList with all the points at (x, y)
	 */
    public LinkedList getAllPointsAt(int x, int y) {
        LinkedList list = new LinkedList();
        for(int i=0;i<pointList.size();i++) {
            if(((GraphPoint) pointList.get(i)).selectedBy(x, y)) {
                list.add(pointList.get(i));
            }
        }
        return list;
    }
    
    /**
	 * Changes the viewing style, i.e. whether squared boxes should 
	 * be drawn or not.
	 */
	public void toggleRelativeSupportLines() {
	    relativeSupportLines = !relativeSupportLines;
	}
	
    /**
	 * Draws support lines in the GraphWindow.
	 * 
	 * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
    public void drawSupportLines(Graphics g, int canvasWidth, int canvasHeight) {
        g.setColor(Color.gray);
        /* Upper line */
        g.drawLine(40, canvasHeight-40, 40+(canvasWidth-80)/2, 40);
        g.drawString(""+maxValue, 50, 50);
        /* Lower line */
        g.drawLine(40, canvasHeight-40, canvasWidth-40, canvasHeight-40-(canvasHeight-80)/2);
        g.drawString(""+maxValue, canvasWidth - 60, canvasHeight-60);
    }
    
    /**
	 * Draws relative support lines in the GraphWindow.
	 * 
	 * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
    public void drawRelativeSupportLines(Graphics g, int canvasWidth,
            int canvasHeight) {
        g.setColor(Color.gray);

        /* Create an array with GraphPoints ranked by x/y*/
        GraphPoint[] xy = new GraphPoint[pointList.size()];
        LinkedList points = (LinkedList) pointList.clone();
        double max = Double.MIN_VALUE;
        int k = 0;
        while(points.size()>0) {
            for(int i=0;i<points.size();i++) {
                GraphPoint gp = (GraphPoint) points.get(i);
                double q = ((gp.getX()-40)*1.0) / ((canvasHeight-40-gp.getY())*1.0);
                if(q>max) {
                    max = q;
                    k = i;
                    
                }
            }
            
            GraphPoint gp = (GraphPoint) points.remove(k);
            xy[xy.length-points.size()-1] = gp;
            max = Double.MIN_VALUE;
        }
        
        /* Calculate lower line */
        double x;
        double y;
        if(xy.length==2) {
            x = (xy[0].getX() + xy[1].getX())/2.0;
            y = (xy[0].getY() + xy[1].getY())/2.0; 
        }
        else {
            int n = xy.length + 1;
            /*  */
            if(n%3==0) {
                x = (xy[xy.length/3+1].getX() + xy[xy.length/3].getX())/2.0;
                y = (xy[xy.length/3+1].getY() + xy[xy.length/3].getY())/2.0;
            }
            /*  */
            else if(xy.length%3==0) {
                x = (xy[xy.length/3-1].getX() + xy[xy.length/3].getX())/2.0;
                y = (xy[xy.length/3-1].getY() + xy[xy.length/3].getY())/2.0;
            }
            /*  */
            else {
                x = (xy[xy.length/3-1].getX() + xy[xy.length/3].getX())/2.0;
                y = (xy[xy.length/3-1].getY() + xy[xy.length/3].getY())/2.0;
            }
            
             
        }
        y = canvasHeight - 40 - y;
        x = x - 40;
        
        /* Draw lower line */
        if((canvasHeight-80)*x/y < canvasWidth-80) {
            g.drawLine(40, canvasHeight-40, (int) ((canvasHeight-80)*x/y+40), 40);
        }
        else {
            g.drawLine(40, canvasHeight-40, canvasWidth-40, ((int) 
                    (canvasHeight-40-(canvasWidth-80)*y/x)));
        }
        
        /* Calculate upper line */
        if(xy.length>2) {
            int n = xy.length + 1;
            /* */
            if(n%3==0) {
                x = (xy[xy.length*2/3-1].getX() + xy[xy.length*2/3].getX())/2.0;
                y = (xy[xy.length*2/3-1].getY() + xy[xy.length*2/3].getY())/2.0;
            }
            /*  */
            else if(xy.length%3==0) {
                x = (xy[xy.length*2/3-1].getX() + xy[xy.length*2/3].getX())/2.0;
                y = (xy[xy.length*2/3-1].getY() + xy[xy.length*2/3].getY())/2.0;
            }
            /*  */
            else {
                x = (xy[xy.length*2/3+1].getX() + xy[xy.length*2/3].getX())/2.0;
                y = (xy[xy.length*2/3+1].getY() + xy[xy.length*2/3].getY())/2.0;
            }
            y = canvasHeight - 40 - y;
            x = x - 40;
        }
        
        /* Draw upper line */
        if((canvasHeight-80)*x/y < canvasWidth-80) {
            g.drawLine(40, canvasHeight-40, (int) ((canvasHeight-80)*x/y+40), 40);
        }
        else {
            g.drawLine(40, canvasHeight-40, canvasWidth-40, ((int) 
                    (canvasHeight-40-(canvasWidth-80)*y/x)));
        }
    }
    
    /**
	 * Sets the size of the drawing area.
	 * 
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
    public void setCanvasSize(int canvasWidth, int canvasHeight) {
	    this.canvasWidth = canvasWidth;
	    this.canvasHeight = canvasHeight;
    }
}
