/*
 * GraphPoint.java
 *
 * Created on 2005-feb-04
 */

package analyzer;

import java.awt.Color;
import java.awt.Graphics;

import rainbowie.Requirement;
import util.Util;

/**
 * In the graph a GraphPoint is the graphical representation of a Requirement.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class GraphPoint {
	
	private Requirement requirement;
	private int x, y;
	private boolean highlighted;
	private Color color = Color.black;
	private int release;
	private boolean hiddenText;
	
	/**
	 * Constructs a GraphPoint for the passed Requirement.
	 * 
	 * @param requirement the requirement for which the GraphPoint will be
	 * created
	 */
	public GraphPoint(Requirement requirement) {
		this.requirement = requirement;
		highlighted = false;
		hiddenText = false;
	}
	
	/**
	 * Returns the x coordinate of the GraphPoint.
	 * 
	 * @return the x coordinate of the GraphPoint
	 */
	public int getX() {
		return x;
	}
	
	/**
	 * Sets the x coordinate for the GraphPoint.
	 * 
	 * @param x the x coordinate
	 */
	public void setX(int x) {
		this.x = x;
	}
	
	/**
	 * Returns the y coordinate of the GraphPoint.
	 * 
	 * @return the y coordinate of the GraphPoint
	 */
	public int getY() {
		return y;
	}
	
	/**
	 * Sets the y coordinate for the GraphPoint.
	 * 
	 * @param y the y coordinate
	 */
	public void setY(int y) {
		this.y = y;
	}
	
	/**
	 * Sets internal release number. The internal release number is used for
	 * assigning different graphical representation for each release.
	 * 
	 * @param release the internal release number
	 */
	public void setRelease(int release) {
	    this.release = release;
	}
	
	/**
	 * Sets whether or not the requirement text sould be hidden in the 
	 * GraphWindow.
	 * 
	 * @param hiddenText set to true to hide the text
	 */
	public void hideText(boolean hiddenText) {
	    this.hiddenText = hiddenText;
	}
	
	/**
	 * Returns the Requirement for this GraphPoint.
	 * 
	 * @return the GraphPoint's Requirement
	 */
	public Requirement getRequirement() {
		return requirement;
	}
	
	/**
	 * Draws the GraphPoint and Requirement text.
	 * 
	 * @param g the Graphics to draw upon
	 */
	public void draw(Graphics g) {
		g.setColor(color);
		if(highlighted) {
			g.setColor(Color.red);
		}
		if(release == -1) {
		    g.drawString("+", getX()-3, getY()+3);
		    if(!hiddenText) {
		        g.drawString(getRequirement().getReqNbr()+": " + 
		                Util.shortString(getRequirement().getReq(), 30, 1),
		                getX(), getY()+10);
		    }
		}
		else {
		    for (int i=0;i<(release+1);i++) {
		        circle(getX(), getY(), 3*(i+1), g);
		    }
		    if(!hiddenText) {
		        g.drawString(getRequirement().getReqNbr()+": " + 
		                Util.shortString(getRequirement().getReq(), 30, 1),
		                getX()+release*2, getY()+17+release*2);
		    }
		}
	}
	
	/**
	 * Each GraphPoint is represented by one or more circles, this method
	 * draws those circles.
	 * 
	 * @param cx the circle's x coordinate
	 * @param cy the circle's y coordinate
	 * @param radius the circle's radius
	 * @param g the Graphics to draw upon
	 */
	private void circle(int cx, int cy, int radius, Graphics g)	{
	    int x, y, p;
	    
	    x = 0;
	    y = radius;
	    p = 1 - radius;
	    
	    while (x <= y) {
	        g.drawLine(cx + x, cy + y, cx + x, cy + y);
	        g.drawLine(cx - x, cy + y, cx - x, cy + y);
	        g.drawLine(cx + x, cy - y, cx + x, cy - y);
	        g.drawLine(cx - x, cy - y, cx - x, cy - y);
	        g.drawLine(cx + y, cy + x, cx + y, cy + x);
	        g.drawLine(cx - y, cy + x, cx - y, cy + x);
	        g.drawLine(cx + y, cy - x, cx + y, cy - x);
	        g.drawLine(cx - y, cy - x, cx - y, cy - x);
	        
	        x = x + 1;
	        if (p < 0) {
	            p = p + 2*x + 1;
	        }
	        else {
	            y = y - 1;
	            p = p + 2*x + 1 - 2*y;
	        }
	    }
	}
	
	/**
	 * Returns true if (xPos, yPos) is within the GraphPoint's outer circle.
	 * 
	 * @param xPos x coordinate
	 * @param yPos y coordinate
	 * @return true if the GraphPoint can be selected from the position 
	 * (xPos, yPos)
	 */
	public boolean selectedBy(int xPos, int yPos) {
	    if (release != -1) {
	        return((xPos>x-6-release*2) && (xPos<x+6+release*2) && 
	                (yPos>y-6-release*2) && (yPos<y+6+release*2));
	    }
	    return((xPos>x-10) && (xPos<x+10) && 
	            (yPos>y-10) && (yPos<y+10));
	}
	
	/**
	 * Highlights the GraphPoint
	 */
	public void highlight() {
		highlighted = true;
	}
	
	/**
	 * Unhighlights the GraphPoint 
	 */
	public void unhighlight() {
		highlighted = false;
	}
	
	/**
	 * Checks whether or not the GraphPoint is highlighted.
	 * 
	 * @return true if GraphPoint is highlighted
	 */
	public boolean isHighLighted() {
	    return highlighted;
	}
	
	/**
	 * Inverts a point in the graph by switching the x-axis with the y-axis.
	 * 
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
	public void invertPoint(int canvasWidth, int canvasHeight) {
	    int oldY = y;
	    int oldX = x;
	    y = canvasHeight-40 - ((oldX-40)*(canvasHeight-80)/(canvasWidth-80));
	    x = 40+(canvasHeight-oldY-40)*(canvasWidth-80)/(canvasHeight-80);
	}
}
