/*
 * PlanningGameAnalyzer.java
 *
 * Created on 2005-feb-04
 */

package analyzer;

import java.awt.Color;
import java.awt.Graphics;
import java.util.LinkedList;
import rainbowie.RequirementList;

/**
 * Defines the analyzer for the Planning Game technique
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class PlanningGameAnalyzer extends Analyzer {
	
    private boolean invertedSupportLines;
    
	/**
     * Constructs a Planning Game analyzer
     * 
     * @param rList		   list of Requirements to be analyzed
     * @param canvasWidth  the width of the drawing area
     * @param canvasHeight the height of the drawing area
     */
	public PlanningGameAnalyzer(RequirementList rList, int canvasWidth, int canvasHeight) {
	    super(rList, canvasWidth, canvasHeight);
	    relativeSupportLines = true;
	    invertedSupportLines = false;
	}
	
	/**
	 * Checks which kind of boxes that are going to be drawn.
	 * 
	 * @return a list containing all the GraphPoints to be drawn
	 */
	public LinkedList createPoints() {
	    if(relativeSupportLines) {
	        return createPointsEqualDist();        
	    }
	    return createPointsSquareBoxes();
	}
	
	/**
     * Creates and assigns the positions of all the requirements to be drawn
     * in the GraphWindow. Method is used when the support lines are drawn 
     * based on the number of requirements in each box.
     * 
     * @return a list containing all the GraphPoints to be drawn
     */
	private LinkedList createPointsEqualDist() {
	    LinkedList oldList = null;
	    if(pointList!=null) {
	        oldList = pointList;
	    }
	    pointList = new LinkedList();
	    
	    int lowsY = 0;
	    int medsY = 0;
	    int highsY = 0;
	    
	    int lowsX = 0;
		int medsX = 0;
		int highsX = 0;
	    
		/* Count nbr of reqs in each pile*/
		for(int i=0;i<rList.getLength();i++) {
			double prioY = rList.getReq(i).getPriority1();
			double prioX = rList.getReq(i).getPriority2();
			if(prioY!=-1.0) {
				if((prioY*10)%10==0.0) {
					highsY++;
				}
				else if((prioY*10)%10==1.0) {
					medsY++;
				}
				else if((prioY*10)%10==2.0) {
					lowsY++;
				}
			}
			if(prioX!=-1.0) {
				if((prioX*10)%10==0.0) {
					highsX++;
				}
				else if((prioX*10)%10==1.0) {
					medsX++;
				}
				else if((prioX*10)%10==2.0) {
					lowsX++;
				}
			}
		}
	    int distX = (canvasWidth-80)/rList.getLength();
	    int distY = (canvasHeight-80)/rList.getLength();
	    GraphPoint gp;
	    for(int i=0;i<rList.getLength();i++) {
	        gp = new GraphPoint(rList.getReq(i));
	        double prioY = rList.getReq(i).getPriority1();
			double prioX = rList.getReq(i).getPriority2();
	        if((prioX*10)%10==2.0) {
	            gp.setX(40 + distX/2 + (lowsX - 1 - ((int) prioX))*distX);
	        }
	        else if((prioX*10)%10==1.0) {
	            gp.setX(40 + distX/2 + (medsX - 1 - (int) prioX)*distX + lowsX*distX);
	        }
	        else if((prioX*10)%10==0.0) {
	            gp.setX(40 + distX/2 + (highsX - 1 - (int) prioX)*distX + (lowsX + medsX)*distX);
	        }
	        
	        if((prioY*10)%10==2.0) {
	            gp.setY(canvasHeight - (40 + distY/2 + (lowsY - 1 -((int) prioY))*distY));
	        }
	        else if((prioY*10)%10==1.0) {
	            gp.setY(canvasHeight - (40 + distY/2 + (medsY - 1 -((int) prioY))*distY + lowsY*distY));
	        }
	        else if((prioY*10)%10==0.0) {
	            gp.setY(canvasHeight - (40 + distY/2 + (highsY - 1 -((int) prioY))*distY + (lowsY + medsY)*distY));
	        }
	        pointList.add(gp);
	    }
	    
	    setReleaseNbrs();
		if(oldList!=null) {
		    for(int i=0;i<oldList.size();i++) {
		        if(((GraphPoint) oldList.get(i)).isHighLighted()) {
		            ((GraphPoint) pointList.get(i)).highlight();
		            selectedPoint = ((GraphPoint) pointList.get(i));
		        }
		    }
		}
	    return pointList;
	}	
	
	/**
     * Creates and assigns the positions of all the requirements to be drawn
     * in the GraphWindow. Method is used when the support lines make 
     * squared boxes.
     * 
     * @return a list containing all the GraphPoints to be drawn
     */
	private LinkedList createPointsSquareBoxes() {
	    LinkedList oldList = null;
	    if(pointList!=null) {
	        oldList = pointList;
	    }
		pointList = new LinkedList();
		int lowsY, medsY, highsY, lowsX, medsX, highsX;
		double prioY, prioX;
		
		/* Vertical */
		lowsY = 0;
		medsY = 0;
		highsY = 0;
		
		/* Horizontal */
		lowsX = 0;
		medsX = 0;
		highsX = 0;
		
		/* Count nbr of reqs in each pile*/
		for(int i=0;i<rList.getLength();i++) {
			prioY = rList.getReq(i).getPriority1();
			prioX = rList.getReq(i).getPriority2();
			if(prioY!=-1.0) {
				if((prioY*10)%10==0.0) {
					highsY++;
				}
				else if((prioY*10)%10==1.0) {
					medsY++;
				}
				else if((prioY*10)%10==2.0) {
					lowsY++;
				}
			}
			if(prioX!=-1.0) {
				if((prioX*10)%10==0.0) {
					highsX++;
				}
				else if((prioX*10)%10==1.0) {
					medsX++;
				}
				else if((prioX*10)%10==2.0) {
					lowsX++;
				}
			}
		}
		int distLowY = ((canvasHeight-80)/3)/(lowsY+1);
		int distMedY = ((canvasHeight-80)/3)/(medsY+1);
		int distHighY = ((canvasHeight-80)/3)/(highsY+1);
		int distLowX = ((canvasWidth-80)/3)/(lowsX+1);
		int distMedX = ((canvasWidth-80)/3)/(medsX+1);
		int distHighX = ((canvasWidth-80)/3)/(highsX+1);
		GraphPoint gp;
		for(int k=0;k<rList.getLength();k++) {
			prioY = rList.getReq(k).getPriority1();
			prioX = rList.getReq(k).getPriority2();
			if(prioY!=-1.0) {
				gp = new GraphPoint(rList.getReq(k));
				if((prioY*10)%10==0.0) {
					gp.setY(((int) prioY + 1)*distHighY + 40);
				}
				else if((prioY*10)%10==1.0) {
					gp.setY(((int) prioY + 1)*distMedY + 40 + (canvasHeight-80)/3);
				}
				else if((prioY*10)%10==2.0) {
					gp.setY(((int) prioY + 1)*distLowY + 40 + 2*(canvasHeight-80)/3);
				}
				
				if((prioX*10)%10==0.0) {
					gp.setX( (canvasWidth-40) - ((int) prioX + 1)*distHighX );
				}
				else if((prioX*10)%10==1.0) {
					gp.setX( (canvasWidth-40) - ((int) prioX + 1)*distMedX - 
					        (canvasWidth-80)/3 );
				}
				else if((prioX*10)%10==2.0) {
					gp.setX( (canvasWidth-40) - ((int) prioX + 1)*distLowX - 
					        2*(canvasWidth-80)/3 );
				}
				pointList.add(gp);
			}			
		}
		setReleaseNbrs();
		if(oldList!=null) {
		    for(int i=0;i<oldList.size();i++) {
		        if(((GraphPoint) oldList.get(i)).isHighLighted()) {
		            ((GraphPoint) pointList.get(i)).highlight();
		            selectedPoint = ((GraphPoint) pointList.get(i));
		        }
		    }
		}
		return pointList;
	}
	
	/**
	 * Draws support lines in the GraphWindow.
	 * 
	 * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
	public void drawSupportLines(Graphics g, int canvasWidth, int canvasHeight) {   
	    g.setColor(Color.gray);
	    /* Upper horizontal */
	    g.drawLine(40, (canvasHeight-80)/3 + 40, canvasWidth-40, 
	            (canvasHeight-80)/3 + 40);
	    /* Lower horizontal */
	    g.drawLine(40, 2*(canvasHeight-80)/3 + 40, canvasWidth-40,
	            2*(canvasHeight-80)/3 + 40);
	    /* Left vertical */
	    g.drawLine( (canvasWidth-80)/3 + 40, canvasHeight-40, 
	            (canvasWidth-80)/3 + 40, 40);
	    /* Right vertical */
	    g.drawLine( 2*(canvasWidth-80)/3 + 40, canvasHeight-40, 
	            2*(canvasWidth-80)/3 + 40, 40);
	}
	
	/**
	 * Draws relative support lines (squares) in the GraphWindow.
	 * 
	 * @param g 		   the Graphics that will be drawn
	 * @param canvasWidth  the width of the drawing area
	 * @param canvasHeight the height of the drawing area
	 */
	public void drawRelativeSupportLines(Graphics g, int canvasWidth, int canvasHeight) {
	    g.setColor(Color.gray);
	    
	    /* Vertical */
	    int lowsY = 0;
	    int medsY = 0;
	    int highsY = 0;
	    
	    /* Horizontal */
	    int lowsX = 0;
	    int medsX = 0;
	    int highsX = 0;
	    
	    /* Count nbr of reqs in each pile*/
	    for(int i=0;i<rList.getLength();i++) {
	        double prioX = 0.0;
	        double prioY = 0.0;
	        if(!invertedSupportLines) {
	            prioY = rList.getReq(i).getPriority1();
	            prioX = rList.getReq(i).getPriority2();
	        }
	        else {
	            prioY = rList.getReq(i).getPriority2();
	            prioX = rList.getReq(i).getPriority1();
	        }
	        if(prioY!=-1.0) {
	            if((prioY*10)%10==0.0) {
	                highsY++;
	            }
	            else if((prioY*10)%10==1.0) {
	                medsY++;
	            }
	            else if((prioY*10)%10==2.0) {
	                lowsY++;
	            }
	        }
	        if(prioX!=-1.0) {
	            if((prioX*10)%10==0.0) {
	                highsX++;
	            }
	            else if((prioX*10)%10==1.0) {
	                medsX++;
	            }
	            else if((prioX*10)%10==2.0) {
	                lowsX++;
	            }
	        }
	    }
	    
	    int distX = (canvasWidth-80)/rList.getLength();
	    int distY = (canvasHeight-80)/rList.getLength();
	    
	    /* Left vertical */
	    if(lowsX!=0) {
	        g.drawLine(40 + distX*lowsX, canvasHeight-40, 
	                40 + distX*lowsX, 40);
	    }
	    else {
	        g.drawLine(45, canvasHeight-40, 45, 40);
	        
	    }
	    
	    /* Right vertical */
	    if(medsX!=0) {
	        g.drawLine(40 + distX*(lowsX+medsX), canvasHeight-40, 
	                40 + distX*(lowsX+medsX), 40);
	    }
	    else {
	        int d = (lowsX==0 ? 50 : 45);
	        g.drawLine(d + distX*lowsX, canvasHeight-40, 
	                d + distX*lowsX, 40);
	    }
	    
	    /* Lower horizontal */
	    if(lowsY!=0) {
	        g.drawLine(40, (canvasHeight-40)-distY*lowsY, canvasWidth-40, 
	                (canvasHeight-40)-distY*lowsY);
	    }
	    else {
	        g.drawLine(40, canvasHeight-45, canvasWidth-40, canvasHeight-45);
	    }
	    
	    /* Upper horizontal */
	    if(medsY!=0) {
	        g.drawLine(40, (canvasHeight-40)-distY*(lowsY+medsY), canvasWidth-40, 
	                (canvasHeight-40)-distY*(lowsY+medsY));
	    }
	    else {
	        int d = (lowsY==0 ? 50 : 45);
	        g.drawLine(40, (canvasHeight-d)-distY*lowsY, canvasWidth-40, 
	                (canvasHeight-d)-distY*lowsY);
	    }  
	}
	
	public void invertSupportLines() {
	    invertedSupportLines = !invertedSupportLines;
	}
}