/*
 * ExcelImporterWindow.java
 *
 * Created on 2005-feb-18
 */

package gui;

import importer.ExcelImporter;
import rainbowie.Rainbowie;
import util.Util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;


/**
 * Defines the window used for importing requirements from an Excel-file.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ExcelImporterWindow extends JFrame {
    
    private JPanel north, center, south, southTop, southBottom;
    private JLabel leftLabel, rightLabel;
    private JButton skipButton, undoButton, cancelButton, acceptButton;
    private JTable excelTable;
    private JTabbedPane tabbedPane;
    private Container contentPane;
    private JMenuItem acceptMenuItem, cancelMenuItem, helpMenuItem;
    private JScrollPane scrollPane;
    private DefaultTableModel excelTableModel;
    
    private int nbrOfRows, nbrOfCols;
    private String[] labelSequence = {"FIRST requirement number", 
            "LAST requirement number", "column containing the requirements",
            "column containing the release numbers" };
    /*
     * indexes is the array containing information about which rows and columns
     * that are selected for importing 
     * indexes = {first row, last row, req. nbr. col, req. col, rel. nbr. col } 
     */
    private int[] indexes = {-1, -1, -1, -1, -1};
    /*
     * counter is used store the current state of the selection process
     * when counter = 0, none of the information in indexes is defined
     */
    private int counter;
    
    private Rainbowie p;
    private ExcelImporter excelImporter;
    private ActionHandler ah;
    private MouseHandler mh;
    private ChangeHandler ch;
    
    /**
     * Constructs an ExcelImporterWindow
     * 
     * @param excelImporter the excelImporter that has started this window
     * @param nbrOfRows		number of rows in the current spreadsheet
     * @param nbrOfCols		number of columns in the current spreadsheet
     * @param p				reference to the programs Parseq object
     */
    public ExcelImporterWindow(ExcelImporter excelImporter, int nbrOfRows,
            int nbrOfCols, Rainbowie p) {
        super("Import");
        this.setSize(800,600);
        this.nbrOfRows = nbrOfRows;
        this.nbrOfCols = nbrOfCols;
        this.excelImporter = excelImporter;
        this.p = p;
        setIconImage(getToolkit().getImage("images\\rainbow.gif"));
        contentPane = this.getContentPane();
        ah = new ActionHandler();
        mh = new MouseHandler();
        ch = new ChangeHandler();
        counter = 0;
        init();
    }
    
    /**
     * Initializes the graphical user interface
     */
    private void init() {
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        
        /* North panel */
		north = new JPanel();
		JLabel header = new JLabel("Requirement Sampler (Excel)");
		Util.setFont(header, true, Util.FONT_MEDIUM);
		north.setBorder(BorderFactory.createEmptyBorder(10,0,10,0));
		north.add(header);
		contentPane.add(north, BorderLayout.NORTH);
		
		/* Center panel */
        center = new JPanel();
        center.setBorder(BorderFactory.createEmptyBorder(10,10,5,10));
        contentPane.add(center, BorderLayout.CENTER);
		center.setLayout(new BorderLayout());
		Object[] colNames = new Object[nbrOfCols];
		Object[][] data = new Object[nbrOfRows][nbrOfCols];
		excelTableModel = new DefaultTableModel(data,	colNames);
		
		/* Set the table and tooltips */
		excelTable = new JTable(excelTableModel) {
	        public Component prepareRenderer(TableCellRenderer renderer,
	                                         int rowIndex, int vColIndex) {
	            Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
	            if (c instanceof JComponent) {
	                JComponent jc = (JComponent)c;
	                if(getValueAt(rowIndex, vColIndex) != null && 
	                        ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
	                    jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
	                }
	                else {
	                    jc.setToolTipText(null);
	                }
	            }
	            return c;
	        }
	    };
	    
		excelTable.setEnabled(false);
		excelTable.setCellSelectionEnabled(true);
		excelTable.addMouseListener(mh);
		excelTable.getTableHeader().setReorderingAllowed(false);
		excelTable.setDefaultRenderer(Object.class, new CustomTableCellRenderer());	
		scrollPane = new JScrollPane(excelTable);
		excelTable.getTableHeader().addMouseListener(mh);
		
		/* Tabbed pane */
		tabbedPane = new JTabbedPane(JTabbedPane.BOTTOM);
		String[] sheets = excelImporter.getSheets();
		tabbedPane.addTab(sheets[0], scrollPane);
		for (int i=1;i<sheets.length;i++) {
		    tabbedPane.addTab(sheets[i], null);
		}
		tabbedPane.addChangeListener(ch);
		center.add(tabbedPane, BorderLayout.CENTER);
		
		/* South panel */
		south = new JPanel();
		south.setLayout(new BorderLayout());
		southTop = new JPanel();
		southBottom = new JPanel();
		southTop.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));
		southBottom.setBorder(BorderFactory.createEmptyBorder(5,10,10,10));
		
		southTop.setLayout(new BoxLayout(southTop, BoxLayout.X_AXIS));
		leftLabel = new JLabel("Double-click on the ");
		rightLabel = new JLabel(labelSequence[0]);
		Util.setFont(leftLabel, true, Util.FONT_MEDIUM);
		Util.setFont(rightLabel, true, Util.FONT_MEDIUM);
		
		southTop.add(leftLabel);
		southTop.add(rightLabel);
				
		skipButton = new JButton("Skip");
		undoButton = new JButton("Undo");
		cancelButton = new JButton("Cancel");
		acceptButton = new JButton("Accept");
		skipButton.setToolTipText("Skip one step");
		undoButton.setToolTipText("Undo last action");
		cancelButton.setToolTipText("Cancel import");
		acceptButton.setToolTipText("Accept and import selected information");
		Util.setJComponentSize(skipButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(undoButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(acceptButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		skipButton.addActionListener(ah);
		skipButton.setEnabled(false);
		undoButton.addActionListener(ah);
		undoButton.setEnabled(false);
		cancelButton.addActionListener(ah);
		acceptButton.setEnabled(false);
		acceptButton.addActionListener(ah);
		southBottom.setLayout(new BoxLayout(southBottom, BoxLayout.X_AXIS));
		southBottom.add(skipButton);
		southBottom.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		southBottom.add(undoButton);
		southBottom.add(Box.createHorizontalGlue());
		southBottom.add(acceptButton);
		southBottom.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		southBottom.add(cancelButton);
		
		south.add(southTop, BorderLayout.NORTH);
		south.add(southBottom, BorderLayout.SOUTH);
		contentPane.add(south, BorderLayout.SOUTH);
		
		/* Menu */
		JMenuBar menuBar = new JMenuBar();
		
		JMenu helpMenu = new JMenu("Help");
		helpMenuItem = new JMenuItem("How to");
		helpMenuItem.addActionListener(ah);
		
		JMenu fileMenu = new JMenu("File");
		acceptMenuItem = new JMenuItem("Accept");
		cancelMenuItem = new JMenuItem("Cancel");
		acceptMenuItem.setEnabled(false);
		acceptMenuItem.addActionListener(ah);
		cancelMenuItem.addActionListener(ah);
		
		menuBar.add(fileMenu);
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		fileMenu.add(acceptMenuItem);
		fileMenu.add(cancelMenuItem);
				
		this.setJMenuBar(menuBar);
		Util.centerJFrame(this);
		setVisible(true);
    }
    
   /**
    * Method for setting tha value in the specified cell in the table model
    * 
    * @param value	the value to set
    * @param row	the row of the specified cell
    * @param column	the column of the specified cell
    */
    public void setCellAt(String value, int row, int column) {
        excelTable.getModel().setValueAt(value, row, column);
    }
    
    /**
     * ChangeListener used for detecting changes in the tabbed pane
     * 
     * @author Mikael Jnsson & Per Klingns
     */
    class ChangeHandler implements ChangeListener {
        
        private int lastSelected = 0;
        
        public void stateChanged(ChangeEvent e) {
            if (e.getSource() == tabbedPane) {
                excelImporter.setSheet(tabbedPane.getSelectedIndex());
                tabbedPane.setComponentAt(lastSelected, new JPanel());
                lastSelected = tabbedPane.getSelectedIndex();
                tabbedPane.setComponentAt(tabbedPane.getSelectedIndex(), scrollPane);
                excelTableModel = new DefaultTableModel(
                        new Object[excelImporter.getRows()][excelImporter.getCols()], 
                        new Object[excelImporter.getCols()]);
                excelTable.setModel(excelTableModel);
                excelImporter.setupExcelImporterWindow();
                for(int i=0;i<indexes.length;i++) {
                    indexes[i] = -1;
                }
                counter = 0;
                undoButton.setEnabled(false);
                skipButton.setEnabled(false);
                acceptButton.setEnabled(false);
                leftLabel.setText("Double-click on the ");
                rightLabel.setText(labelSequence[counter]);
            }
        }
    }
    
    /**
     * ActionListener to handle user interaction.
     * 
     * @author Mikael Jnsson & Per Klingns
     */
    class ActionHandler implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            if(e.getSource() == skipButton) {
                counter = counter + 1;
                if(counter>3) {
                    counter = 4;
                    leftLabel.setText("");
                    rightLabel.setText("Press \"Accept\" to import selected requirements");
                    skipButton.setEnabled(false);
                }
                else {
                    rightLabel.setText(labelSequence[counter]);
                }
            }
            else if(e.getSource() == undoButton) {
                counter = counter - 1;
                switch(counter) {
                case 0 :
                    indexes[0]= -1;
            	    indexes[2]= -1;
            	    skipButton.setEnabled(false);
            	    undoButton.setEnabled(false);
            	    break;
                case 1 :
                    indexes[1] = -1;
                    skipButton.setEnabled(false);
            	    acceptButton.setEnabled(false);
            	    acceptMenuItem.setEnabled(false);
                    break;
                case 2 :
                    indexes[3] = -1;
            	    skipButton.setEnabled(true);
            	    break;
                case 3 :
                    indexes[4] = -1;
                    skipButton.setEnabled(true);
                    leftLabel.setText("Double-click on the ");
                    break;
                }
                rightLabel.setText(labelSequence[counter]);
                excelTable.repaint();
            }
            else if(e.getSource() == cancelButton || e.getSource() == cancelMenuItem) {
                dispose();
            }
            else if(e.getSource() == acceptButton || e.getSource() == acceptMenuItem) {
                excelImporter.importRequirements(((DefaultTableModel) excelTable.getModel()).getDataVector(), indexes);
                dispose();
            }
            else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("excelimporterwindow.html");
			}
        }
    }
    
    /**
     * MouseListener to handle user interaction
     * 
     * @author Mikael Jnsson & Per Klingns
     */
    class MouseHandler implements MouseListener {
        
        public void mouseClicked(MouseEvent e) {
            if( (e.getSource() == excelTable || 
                    e.getSource() == excelTable.getTableHeader()) 
                    && e.getClickCount() == 2) {
                int row = excelTable.rowAtPoint(e.getPoint());
                int col = excelTable.columnAtPoint(e.getPoint());
                switch(counter) {
                	case 0 :
                	    indexes[0] = row;
                	    indexes[2] = col;
                	    counter++;
                	    rightLabel.setText(labelSequence[counter]);
                	    skipButton.setEnabled(false);
                	    undoButton.setEnabled(true);
                	    break;
                	case 1 :
                	    if(col==indexes[2] && row>=indexes[0]) {
                	        indexes[1] = row;
                	        counter++;
                	        rightLabel.setText(labelSequence[counter]);
                	        skipButton.setEnabled(true);
                	        acceptButton.setEnabled(true);
                	        acceptMenuItem.setEnabled(true);
                	    }
                	    else if(col!=indexes[2]){
                	        JOptionPane.showMessageDialog(null, 
                	                "Column mismatch with previous selection",
                					"Error", JOptionPane.WARNING_MESSAGE);
                	    }
                	    else {
                	        JOptionPane.showMessageDialog(null, 
                	                "Last requirement number cannot be\n" +
                	                "above first requirement number",
                					"Error", JOptionPane.WARNING_MESSAGE);
                	    }
                	    break;
                	case 2 :
                	    indexes[3] = col;
                	    counter++;
                	    skipButton.setEnabled(true);
                	    rightLabel.setText(labelSequence[counter]);
                	    break;
                	case 3 :
                	    indexes[4] = col;
                	    counter++;
                	    skipButton.setEnabled(false);
                	    leftLabel.setText("");
                	    rightLabel.setText("Press \"Accept\" to import selected requirements");
                	    break;           	
                }
                excelTable.repaint();
           }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}  
    }
    
    /**
     * DefaultTableCellRenderer used for setting the 
     * color marking of the selected cells
     * 
     * @author Mikael Jnsson & Per Klingns
     */
    public class CustomTableCellRenderer extends DefaultTableCellRenderer {
        
        private Color selectedColor = SystemColor.activeCaption ;
        private Color selectedText = SystemColor.textHighlightText;
        private Color unselectedText = SystemColor.textText;
        
        /**
         * Overridden method to make it possible to color mark
         * the selected cells
         */
        public Component getTableCellRendererComponent
        (JTable table, Object value, boolean isSelected,
                boolean hasFocus, int row, int column) {
            
            Component cell = super.getTableCellRendererComponent
            (table, value, isSelected, hasFocus, row, column);
            setOpaque(true);
            
            /* Requirement number */
            if(column == indexes[2]) {
                if((row == indexes[0]) || 
                        (row > indexes[0] && row < indexes[1]+1)) {
                    cell.setBackground(selectedColor);
                    cell.setForeground(selectedText);
                }
            }
            
            /* Requirement */
            else if(column == indexes[3]){
                if((row == indexes[0]) || 
                        (row > indexes[0] && row < indexes[1]+1)) {
                    cell.setBackground(selectedColor);
                    cell.setForeground(selectedText);
                }
                else {
                    cell.setBackground(Color.white);
                    cell.setForeground(unselectedText);
                }
            }
            /* Release number */
            else if(column == indexes[4]){
                if((row == indexes[0]) || 
                        (row > indexes[0] && row < indexes[1]+1)) {
                    cell.setBackground(selectedColor);
                    cell.setForeground(selectedText);
                }
                else {
                    cell.setBackground(Color.white);
                    cell.setForeground(unselectedText);
                }
            }
            else {
                cell.setBackground(Color.white);
                cell.setForeground(unselectedText);
            }            
            return cell;
            
        }
    }
}
