/*
 * GraphWindow.java
 *
 * Created on 2005-feb-03
 */

package gui;


import javax.imageio.ImageIO;
import javax.swing.*;

import analyzer.*;
import rainbowie.*;
import util.Filter;
import util.Util;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;


/**
 * Defines the window used to visualize the results from the
 * prioritization in a graph
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class GraphWindow extends JFrame {
	private ActionHandler ah;
	private Analyzer analyzer;
	
	private Container contentPane;
	private JButton invertButton, saveButton, 
		matrixButton, closeButton, addButton;
	private GraphCanvas graphCanvas;
	private ImprovementsWindow iw;
	private MouseHandler mh;
	private JPanel north, south, east;
	private Rainbowie p;
	private LinkedList pointList;
	private LinkedList dependenciesList;
	private RequirementList rList;
	private boolean inverted;
	private JTextArea infoArea;
	private JScrollPane infoScroll;
	private JMenuItem closeMenuItem, saveGraphMenuItem, invertGraphMenuItem,
	addMenuItem, showMatrixMenuItem, helpMenuItem;
	private JCheckBoxMenuItem hideReqTextMenuItem, hideDepTextMenuItem, hideDepMenuItem, 
	changeViewMenuItem;
	
	/**
	 * Construcst a GraphWindow
	 * 
	 * @param rList the ReequirementList containing the requirements
	 * to be displayed
	 * @param p reference to the programs Parseq object
	 * @param windowNbr Window identifier that shows which Root-Cause Matrix 
	 * that belong to this window
	 */
	public GraphWindow(RequirementList rList, Rainbowie p, int windowNbr) {
		super("Graph #" + windowNbr);
		this.rList = rList;
		this.p = p;
		inverted = false;
		dependenciesList = new LinkedList();
		
		ah = new ActionHandler();
		mh = new MouseHandler();
		iw = new ImprovementsWindow(p, this, windowNbr);
		iw.setVisible(false);
		
		init();
		
	}
	 
	/**
	 * Initializes the graphical user interface
	 */
	private void init() {
		contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		this.setResizable(true);
		Dimension screenRes = Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize(screenRes.width-screenRes.width/4, screenRes.height-screenRes.height/4);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		
		/* North panel */
		north = new JPanel();
		JLabel header = new JLabel("Requirement graph");
		Util.setFont(header, true, Util.FONT_MEDIUM);
		north.add(header);
		contentPane.add(north, BorderLayout.NORTH);
		
		/* West panel */
		JPanel west = new JPanel();
		west.setBorder(BorderFactory.createEmptyBorder(0,0,0,10));
		contentPane.add(west, BorderLayout.WEST);
		
		/* Center panel */
		graphCanvas = new GraphCanvas();
		graphCanvas.addMouseListener(mh);
		graphCanvas.addMouseMotionListener(mh);
		JScrollPane scrollPane = new JScrollPane(graphCanvas);
		
		contentPane.add(scrollPane, BorderLayout.CENTER);
		
		/* South panel */
		south = new JPanel();
		south.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
		contentPane.add(south, BorderLayout.SOUTH);
		
		/* East panel */
		east = new JPanel();
		east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
		east.setBorder(BorderFactory.createEmptyBorder(0, 30, 4, 20) );
		
		infoArea = new JTextArea("Requirement #: " + "\n" + 
	    	    "\n\n" + "Release #: " + "\n" + 
	    	    "\n\n" + "Requirement: " + "\n");
		infoArea.setLineWrap(true);
		infoArea.setWrapStyleWord(true);
		Util.setFont(infoArea, false, Util.FONT_XSMALL);
		infoArea.setEditable(false);
		infoScroll = new JScrollPane(infoArea);
		Util.setJComponentSize(infoScroll, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT*8);
		infoScroll.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		invertButton = new JButton("Invert Graph");
		Util.setJComponentSize(invertButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		invertButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		invertButton.addActionListener(ah);
		invertButton.setToolTipText("Invert the axes of the graph");
		saveButton = new JButton("Save Graph");
		Util.setJComponentSize(saveButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		saveButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		saveButton.addActionListener(ah);
		saveButton.setToolTipText("Save the graph to a png-file");
		addButton = new JButton("Add to Matrix");
		Util.setJComponentSize(addButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		addButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		addButton.setEnabled(false);
		addButton.addActionListener(ah);
		addButton.setToolTipText("Add the selected requirement to the root-cause matrx");
		matrixButton = new JButton("Show Matrix");
		Util.setJComponentSize(matrixButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		matrixButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		matrixButton.setEnabled(true);
		matrixButton.addActionListener(ah);
		matrixButton.setToolTipText("Show the root-cause matrix");
		closeButton = new JButton("Close");
		Util.setJComponentSize(closeButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		closeButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		closeButton.addActionListener(ah);
		closeButton.setToolTipText("Close this window");
		
		east.add(infoScroll);
		east.add(Box.createVerticalGlue());
		east.add(invertButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		east.add(saveButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		east.add(addButton);
		east.add(Box.createVerticalStrut(Util.VERTICAL_BUTTON_DISTANCE));
		east.add(matrixButton);
		east.add(Box.createVerticalStrut(40));
		east.add(closeButton);
		east.add(Box.createVerticalStrut(-3));
		contentPane.add(east, BorderLayout.EAST);
		
		/* Menu */
		JMenuBar menuBar;
		JMenu fileMenu, editMenu, helpMenu;
		
		menuBar = new JMenuBar();
		fileMenu = new JMenu("File");
		editMenu = new JMenu("Edit");
		helpMenu = new JMenu("Help");
		
		saveGraphMenuItem = new JMenuItem("Save Graph");
		saveGraphMenuItem.addActionListener(ah);
		closeMenuItem = new JMenuItem("Close");
		closeMenuItem.addActionListener(ah);
		
		invertGraphMenuItem = new JMenuItem("Invert Graph");
		invertGraphMenuItem.addActionListener(ah);
		hideReqTextMenuItem = new JCheckBoxMenuItem("Hide Requirement Texts");
		hideReqTextMenuItem.addActionListener(ah);
		hideDepTextMenuItem = new JCheckBoxMenuItem("Hide Dependency Texts");
		hideDepTextMenuItem.addActionListener(ah);
		hideDepMenuItem = new JCheckBoxMenuItem("Hide Dependencies");
		hideDepMenuItem.addActionListener(ah);
		addMenuItem = new JMenuItem("Add to Matrix");
		addMenuItem.setEnabled(false);
		addMenuItem.addActionListener(ah);
		showMatrixMenuItem = new JMenuItem("Show Matrix");
		showMatrixMenuItem.setEnabled(true);
		showMatrixMenuItem.addActionListener(ah);
		changeViewMenuItem = new JCheckBoxMenuItem("Change View");
		changeViewMenuItem.addActionListener(ah);
		changeViewMenuItem.setEnabled(true);
		
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ah);
		
		menuBar.add(fileMenu);
		fileMenu.add(saveGraphMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(closeMenuItem);
		
		menuBar.add(editMenu);
		editMenu.add(invertGraphMenuItem);
		editMenu.add(hideDepMenuItem);
		editMenu.add(hideDepTextMenuItem);
		editMenu.add(hideReqTextMenuItem);		
		editMenu.add(changeViewMenuItem);
		editMenu.addSeparator();
		editMenu.add(addMenuItem);
		editMenu.add(showMatrixMenuItem);
		
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		
		this.setJMenuBar(menuBar);
		Util.centerJFrame(this);
		setVisible(true);
		createPoints();
		
		/* ComponentListener to set a minimum allowed window size */
		this.addComponentListener(new java.awt.event.ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				JFrame tmp = (JFrame)e.getSource();
				int width = tmp.getWidth();
				int height = tmp.getHeight();
				if (width<800) {
					width = 800;
				}
				if(height<600) {
					height = 650;
				}
				tmp.setSize(width, height);
				repaint();
			}
		});
	}
	
	/**
	 * Checks used prio method and constructs the corresponding analyzer object
	 * Sets the list of points to be drawn in the graph
	 */
	private void createPoints() {
		if(rList.getPrioMethod() == RequirementList.PLANNING_GAME) {
		    analyzer = new PlanningGameAnalyzer(rList, graphCanvas.getWidth(),
		            graphCanvas.getHeight());
		}
		else if(rList.getPrioMethod() == RequirementList.DOLLAR) {
		    analyzer = new DollarTechniqueAnalyzer(rList, graphCanvas.getWidth(),
		            graphCanvas.getHeight());
		}
		else if(rList.getPrioMethod() == RequirementList.PAIR_WISE) {
		    analyzer = new PairwiseAnalyzer(rList, graphCanvas.getWidth(),
		            graphCanvas.getHeight());
		}
		pointList = analyzer.createPoints();
	}

	/**
	 * Gets an image of the GraphCanvas. 
	 * 
	 * @return an image of the graph
	 */
	public Image getGraphImage() {
	    analyzer.selectPoint(-1,-1);
	    Image image = graphCanvas.createImage(graphCanvas.getWidth(), graphCanvas.getHeight());
	    Graphics g = image.getGraphics();
	    graphCanvas.paint(g);
	    
	    return image;
	}
	
	/**
	 * Opens a confirm dialog that alerts the user to the fact that closing 
	 * this window will close both this and the Root-Cause Matrix window. If 
	 * the user chooses to close the windows, both windows are disposed. 
	 */
	public void dispose() {
	    int n = JOptionPane.showConfirmDialog(null, 
                "Closing this window will also close the Root-Cause\n" +
                "Matrix window if it has been opened.\n" +
				  "All unsaved changes will be lost.\n" +
                "\n" +
                "Close the Graph and the Root-Cause Matrix Windows?\n",
                "Close Windows?", JOptionPane.YES_NO_OPTION);
        if(n == JOptionPane.YES_OPTION) {
            iw.dispose();
            super.dispose();
        }
	    
	    
	}
	
	/**
	 * Sets the information text in the info area
	 * 
	 * @param r the current Requirement which information shall be displayed
	 */
	private void setInfoAreaText(Requirement r) {  
	    String s = "Requirement #: " + "\n" + r.getReqNbr() + 
	    "\n\n" + "Release #: " + "\n" + r.getRelNbr() + 
	    "\n\n" + "Requirement: " + "\n" + r.getReq();
		infoArea.setText(s);
		infoArea.setCaretPosition(0);
	}
	
	/**
     * ActionListener to handle user interaction.
     */
	class ActionHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == closeButton || e.getSource() == closeMenuItem) {
				dispose();
			}
			else if(e.getSource() == invertButton || e.getSource() == invertGraphMenuItem)  {
			    inverted = !inverted;
			    if (analyzer instanceof PlanningGameAnalyzer) {
			        ((PlanningGameAnalyzer) analyzer).invertSupportLines();
			    }
			    graphCanvas.repaint();
			}
			else if(e.getSource() == saveButton || e.getSource() == saveGraphMenuItem) {
			    analyzer.selectPoint(-1,-1);	// deselect the selected graph point
			    JFileChooser chooser = new JFileChooser();
			    chooser.addChoosableFileFilter(new Filter("png"));
				int returnVal = chooser.showSaveDialog(null);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File oldFile = chooser.getSelectedFile();
                    
                    int response = JOptionPane.OK_OPTION;
                    if(oldFile.exists()) {
                        response = JOptionPane.showConfirmDialog (null,
                                "Overwrite existing file?","Confirm Overwrite",
                                JOptionPane.OK_CANCEL_OPTION,
                                JOptionPane.QUESTION_MESSAGE);
                    } 
                    if (response == JOptionPane.OK_OPTION) {
                        File newFile;
                        Image image = graphCanvas.createImage(graphCanvas.getWidth(), 
                                graphCanvas.getHeight());
                        if(!oldFile.getName().endsWith(".png")) {
                            newFile = new File(oldFile.getAbsolutePath()+".png");
                        } else {
                            newFile = oldFile;
                        }
                        Graphics g = image.getGraphics();
                        graphCanvas.paint(g);
                        try {
                            ImageIO.write((RenderedImage)image, "png", newFile);
                        }
                        catch(IOException ioe) {
                            JOptionPane.showMessageDialog(null, "Unable to save graph",
                                    "Failed", JOptionPane.ERROR_MESSAGE);
                        }
                    }
				}
			}
			else if(e.getSource() == addButton || e.getSource() == addMenuItem) {
			    Requirement r = analyzer.getSelectedPointsRequirement();
			    if (r != null) {
			        iw.addReq(r);
			        matrixButton.setEnabled(true);
			        showMatrixMenuItem.setEnabled(true);
			    }
			}
			else if (e.getSource() == matrixButton || e.getSource() == showMatrixMenuItem){
			    iw.dispose();
			    iw.setVisible(true);
			    iw.invalidate();
			    iw.validate();
			    iw.setState(JFrame.NORMAL);
			    iw.requestFocus();
			}
			else if (e.getSource() == hideReqTextMenuItem) {
			    graphCanvas.repaint();
			}
			else if (e.getSource() == hideDepTextMenuItem) {
			    graphCanvas.repaint();
			}
			else if (e.getSource() == hideDepMenuItem) {
			    graphCanvas.repaint();
			}
			else if(e.getSource() == changeViewMenuItem) {
			    analyzer.toggleRelativeSupportLines();
			}
			else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("graphwindow.html");
			}
		}
	}

	/**
	 * The drawing area where the graph is drawn
	 */
	class GraphCanvas extends JComponent {
		
	    private GraphPoint[] dragLine = null;
	    
	    /**
	     * Constructs a GraphCanvas
	     */
		public GraphCanvas() {
		    super();
		}
		
		/**
		 * Draws the X- and Y-axes
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawAxes(Graphics g) {
			
			/* Vertical line */
		    g.setColor(Color.lightGray);
			g.drawLine(39, 20, 39, getHeight()-20);
		    g.setColor(Color.black);
			g.drawLine(40, 20, 40, getHeight()-20);
			g.setColor(Color.gray);
			g.drawLine(41, 20, 41, getHeight()-20);
			/* Horizontal line */
			g.setColor(Color.lightGray);
			g.drawLine(20, getHeight()-39, getWidth()-20, getHeight()-39);
			g.setColor(Color.black);
			g.drawLine(20, getHeight()-40, getWidth()-20, getHeight()-40);
			g.setColor(Color.gray);
			g.drawLine(20, getHeight()-41, getWidth()-20, getHeight()-41);
			
			/* Top arrow */
			g.setColor(Color.black);
			int[] xt = {30,40,50};
			int[] yt = {30,15,30};
			g.fillPolygon(xt,yt,3);
			g.setColor(Color.gray);
			g.drawLine(xt[0], yt[0], xt[1], yt[1]);
			g.drawLine(xt[1], yt[1]+1, xt[2], yt[2]);
			g.drawLine(xt[2], yt[2], xt[0], yt[0]);
			
			/* Right arrow */
			g.setColor(Color.black);
			int[] xr = { getWidth()-30, getWidth()-15, getWidth()-30 };
			int[] yr = { getHeight()-50, getHeight()-40, getHeight()-30 };
			g.fillPolygon(xr,yr,3);
			g.setColor(Color.gray);
			g.drawLine(xr[0], yr[0], xr[1], yr[1]);
			g.drawLine(xr[1], yr[1], xr[2], yr[2]);
			g.drawLine(xr[2], yr[2], xr[0], yr[0]);
			
			g.setColor(Color.black);
			
			/* The labels for the axes */
			if(!inverted) {
			    g.drawString(rList.getPrioCriteria1(), 60, 20);
			    g.drawString(rList.getPrioCriteria2(), getWidth()-70, getHeight()-20);
			}
			else {
			    g.drawString(rList.getPrioCriteria2(), 60, 20);
			    g.drawString(rList.getPrioCriteria1(), getWidth()-70, getHeight()-20);
			}
		}
		
		/**
		 * Draws the graph points 
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawPoints(Graphics g) {
		    int k = -1;
			for(int i=0;i<pointList.size();i++) {
				((GraphPoint) pointList.get(i)).draw(g);
				k = (((GraphPoint) pointList.get(i)).isHighLighted() ? i : k);
			}
			/* Draw the selected point an extra time */
			if(k!=-1) {
			    ((GraphPoint) pointList.get(k)).draw(g);
			}
		}
		
		/**
		 * Draws the support lines for the specific graph type
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawSupportLines(Graphics g) {
		    analyzer.drawSupportLines(g, getWidth(), getHeight());
		}
		
		/**
		 * Draws the relative support lines for the specific graph type
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawRelativeSupportLines(Graphics g) {
		    analyzer.drawRelativeSupportLines(g, getWidth(), getHeight());
		}
		
		/**
		 * Draws the user defined dependency lines
		 * 
		 * @param g the Graphics object to draw on
		 */
		private void drawDependencies(Graphics g) {
		    g.setColor(new Color(125, 125, 255));
		    if(dragLine!=null) {
		        g.drawLine(dragLine[0].getX(), dragLine[0].getY(),
		                dragLine[1].getX(), dragLine[1].getY());
		    }
		    for(int i=0;i<dependenciesList.size();i++) {
		        if(!hideDepMenuItem.isSelected()) {
		            int fromX = ((GraphPoint) ((Object[]) dependenciesList.get(i))[0]).getX();
		            int fromY = ((GraphPoint) ((Object[]) dependenciesList.get(i))[0]).getY();
		            int toX = ((GraphPoint) ((Object[]) dependenciesList.get(i))[1]).getX();
		            int toY = ((GraphPoint) ((Object[]) dependenciesList.get(i))[1]).getY();
		            g.drawLine(fromX, fromY, toX, toY);
		            if(!hideDepTextMenuItem.isSelected()) {
		                if((fromX<toX && fromY>toY) || (fromX>toY && fromY<toY)) {
		                    g.drawString((String) ((Object[]) dependenciesList.get(i))[2], (fromX+toX)/2+5, (fromY+toY)/2+5);
		                }
		                else {
		                    g.drawString((String) ((Object[]) dependenciesList.get(i))[2], (fromX+toX)/2-5, (fromY+toY)/2-5);
		                }
		            }
		        }
		    }
		}
		
		/**
		 * Method to do all the painting
		 */
		public void paint(Graphics g) {
		    
			g.setColor(Color.white);
			g.fillRect(0, 0, getWidth(), getHeight());
			drawAxes(g);
			if(analyzer!=null) {
			    analyzer.setCanvasSize(getWidth(), getHeight());
			    pointList = analyzer.createPoints();
			    if(inverted) {
			        for(int i=0;i<pointList.size();i++) {
			            ((GraphPoint) pointList.get(i)).
			            invertPoint(graphCanvas.getWidth(), graphCanvas.getHeight());
			            
			        }
			    }
			    for(int i=0;i<pointList.size();i++) {
			        GraphPoint gp = (GraphPoint) pointList.get(i);
			        /* Hide text if Hide text button is pressed */
			        gp.hideText(hideReqTextMenuItem.isSelected());
			        
			        /* 
			         * Update points in the dependencies list, needed for inverting
			         * and reinverting
			         */
			        for(int k=0;k<dependenciesList.size();k++) {
		                Object[] points =  (Object[]) dependenciesList.get(k);
		                if(((GraphPoint) points[0]).getRequirement().equals(gp.getRequirement())) {
		                    points[0] = gp;
		                }
		                if(((GraphPoint) points[1]).getRequirement().equals(gp.getRequirement())) {
		                    points[1] = gp;
		                }
		            }
			    }
			    drawPoints(g);
			    if( (changeViewMenuItem.isSelected() && !(analyzer instanceof PlanningGameAnalyzer)) || 
			            (analyzer instanceof PlanningGameAnalyzer && !changeViewMenuItem.isSelected()) ) {
			        drawRelativeSupportLines(g);
			    }
			    else {
			        drawSupportLines(g);
			    }
			    drawDependencies(g);
			}
		}
	}
	
	/**
	 * MouseListener to handle user interaction
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class MouseHandler implements MouseListener, MouseMotionListener  {
	    private Object[] currentDependency;
	    private LinkedList currentDependencyFromList;

		public void mouseClicked(MouseEvent e) {
			if(e.getClickCount()==1 && e.getSource() == graphCanvas) {
			    /* Check if more than one point is selected */
			    LinkedList list = analyzer.getAllPointsAt(e.getX(), e.getY());
			    Object[] options = new Object[list.size()];
			    Requirement r = null;
			    /* If more than one point is selected, display a JOptionPane */
			    if(list.size()>1) {
			        for(int i=0;i<list.size();i++) {
				        options[i] = ((GraphPoint) list.get(i)).getRequirement().getReqNbr();
				    }
			        Object selectedValue = JOptionPane.showInputDialog(null,
							"Choose the requirement to be selected:", 
							"Overlapping requirements",
							JOptionPane.INFORMATION_MESSAGE, null, options,
							options[0]);
			        if ( selectedValue != null ) {
			            for(int i=0;i<list.size();i++) {
			                if(((GraphPoint) list.get(i)).getRequirement().getReqNbr().compareTo(
			                        selectedValue.toString())==0) {
			                    r = ((GraphPoint) list.get(i)).getRequirement();
			                    analyzer.highlightPoint(r);
			                }
			            }
			        }
			    }
			    else {
			        r = analyzer.selectPoint(e.getX(), e.getY());
			    }
				if (r != null) {
					setInfoAreaText(r);
					addButton.setEnabled(true);
					addMenuItem.setEnabled(true);
				}
				else {
				    infoArea.setText("Requirement #: " + "\n" + 
				    	    "\n\n" + "Release #: " + "\n" + 
				    	    "\n\n" + "Requirement: " + "\n");
				    addButton.setEnabled(false);
				    addMenuItem.setEnabled(false);
				}
				graphCanvas.repaint();
			}
			else if(e.getClickCount()==2 && e.getSource() == graphCanvas) {
				Requirement r = analyzer.selectPoint(e.getX(), e.getY());
				if (r != null) {
					iw.addReq(analyzer.selectPoint(e.getX(), e.getY()));
					matrixButton.setEnabled(true);
					showMatrixMenuItem.setEnabled(true);
				}
			}
		}
		
		public void mouseEntered(MouseEvent e) {}
		
		public void mouseExited(MouseEvent e) {}
		
		public void mousePressed(MouseEvent e) {
		    if(e.getSource()==graphCanvas && e.isShiftDown()) {
		        LinkedList list = analyzer.getAllPointsAt(e.getX(), e.getY());
		        if(list.size()>0) {
		            if(list.size()>1) {
		                currentDependencyFromList = list;
		            }
		            currentDependency = new Object[3];
		            currentDependency[0] = (GraphPoint) list.get(0);
		        }
		    }
		}
		
		public void mouseReleased(MouseEvent e) {
		    if(e.getSource()==graphCanvas) {
		        boolean addOk = true;
		        LinkedList list = analyzer.getAllPointsAt(e.getX(), e.getY());
		        if(list.size()>0 && currentDependencyFromList!=null) {
		            Object[] options = new Object[currentDependencyFromList.size()];
	                for(int i=0;i<currentDependencyFromList.size();i++) {
	                    options[i] = ((GraphPoint) currentDependencyFromList.get(i)).getRequirement().getReqNbr();
	                }
	                Object selectedValue = JOptionPane.showInputDialog(null,
	                        "Choose the requirement to assign\n" +
	                        "the dependency with:", 
	                        "Overlapping requirments",
	                        JOptionPane.INFORMATION_MESSAGE, null, options,
	                        options[0]);
	                if (selectedValue != null) {
	                    for(int i=0;i<currentDependencyFromList.size();i++) {
	                        if(((GraphPoint) currentDependencyFromList.get(i)).getRequirement().getReqNbr().compareTo(
	                                selectedValue.toString())==0) {
	                            currentDependency[0] = (GraphPoint) currentDependencyFromList.get(i);
	                        }
	                    }
	                }
	                else {
	                    currentDependency = null;
	                }
		        }
		        
		        if(list.size()>0 && currentDependency!=null) {
		            if(list.size()>1) {
		                Object[] options = new Object[list.size()];
		                for(int i=0;i<list.size();i++) {
		                    options[i] = ((GraphPoint) list.get(i)).getRequirement().getReqNbr();
		                }
		                Object selectedValue = JOptionPane.showInputDialog(null,
		                        "Choose the requirement to assign\n" +
		                        "the dependency with:", 
		                        "Overlapping requirements",
		                        JOptionPane.INFORMATION_MESSAGE, null, options,
		                        options[0]);
		                if (selectedValue != null) {
		                    for(int i=0;i<list.size();i++) {
		                        if(((GraphPoint) list.get(i)).getRequirement().getReqNbr().compareTo(
		                                selectedValue.toString())==0) {
		                            currentDependency[1] = (GraphPoint) list.get(i);
		                        }
		                    }
		                }
		                else {
		                    /* To avoid nullpointer */
		                    currentDependency[1] = (GraphPoint) list.get(0);
		                    addOk = false;
		                }
		            }
		            else {
		                currentDependency[1] = (GraphPoint) list.get(0);
		            }
		            
		            
		            for(int i=0;i<dependenciesList.size();i++) {
		                Object[] p = (Object[]) dependenciesList.get(i);
		                /* Remove a dependency */
		                if((((GraphPoint) p[0]).getRequirement().equals(((GraphPoint) currentDependency[0]).getRequirement()) &&
		                        ((GraphPoint)p[1]).getRequirement().equals(((GraphPoint) currentDependency[1]).getRequirement())) ||
		                        (((GraphPoint)p[0]).getRequirement().equals(((GraphPoint) currentDependency[1]).getRequirement()) &&
		                                ((GraphPoint)p[1]).getRequirement().equals(((GraphPoint) currentDependency[0]).getRequirement()))) {
		                    dependenciesList.remove(i);
		                    addOk = false;
		                }
		            }
		            if(addOk && !((GraphPoint) currentDependency[0]).getRequirement().
		                    equals(((GraphPoint) currentDependency[1]).getRequirement())) {
		                
		                String depDesc = JOptionPane.showInputDialog(graphCanvas, "Enter the description for this dependency", 
		                        "Dependency description", JOptionPane.QUESTION_MESSAGE);
		                if (depDesc != null) {
		                    currentDependency[2] = depDesc;
		                    dependenciesList.add(currentDependency);
		                }
		            }
		        }
		        currentDependencyFromList = null;
		        currentDependency = null;
		        graphCanvas.dragLine = null;
		        repaint();
		    }
		}
	    
		public void mouseDragged(MouseEvent e) {
		    if(e.getSource()==graphCanvas && currentDependency!=null) {
		        if(graphCanvas.dragLine==null) {
		            graphCanvas.dragLine = new GraphPoint[2];
		        }
		        graphCanvas.dragLine[0] = (GraphPoint) currentDependency[0];
		        GraphPoint tempPoint = new GraphPoint(null);
		        tempPoint.setX(e.getX());
		        tempPoint.setY(e.getY());
		        graphCanvas.dragLine[1] = tempPoint;
		        graphCanvas.repaint();
		    }
		}
		
		public void mouseMoved(MouseEvent e) {
		    LinkedList list = analyzer.getAllPointsAt(e.getX(), e.getY());
		    if(list.size()==1) {
		        Requirement r = ((GraphPoint) list.getFirst()).getRequirement();
		        graphCanvas.setToolTipText(r.getReqNbr()+": "+r.getReq());
		    } 
		    else if (list.size()>1) {
		        graphCanvas.setToolTipText("Overlapping requirements");
	        }
		    else {
		        graphCanvas.setToolTipText(null);
		    }
		}
	}	
}
