/*
 * ImportWindow.java
 *
 * Created on Jan 21, 2005
 */

package gui;

import importer.ExcelImporter;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import rainbowie.Rainbowie;
import rainbowie.RequirementList;
import util.Filter;
import util.Util;


/**
 * Defines the window used for importing requirements.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ImportWindow extends JFrame {


	private JButton openButton, acceptButton, editRowButton, cancelButton;
	
	private Container contentPane;
	
	private KeyHandler kh;
	private MouseHandler mh;
	private ActionHandler ah;
	
	private JPanel northPanel, centerPanel;
	
	private Rainbowie p;
	private EditWindow ew;

	private JMenuItem cancelMenuItem, helpMenuItem,
					acceptMenuItem, openMenuItem,
					editRowMenuItem, clearTableMenuItem;

	private JTable reqTable;
	private boolean[] markedRows = new boolean[1];

	/**
	 * Constructs an ImportWindow
	 * 
	 * @param p reference to the programs Parseq object
	 */
	public ImportWindow(Rainbowie p) {
	    super("Import Window");
		this.p = p;
		init();
	}
	
	/**
	 * Initializes the graphical user interface
	 */
	private void init() {
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(680, 400);
		Util.centerJFrame(this);
		this.setResizable(true);
		setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		mh = new MouseHandler();
		ah = new ActionHandler(this);
		ew = new EditWindow();
		kh = new KeyHandler();
		this.getContentPane().setFocusable(true);
		this.getContentPane().addKeyListener(kh);
		this.getContentPane().requestFocusInWindow();
		
		/* North panel */
		northPanel = new JPanel();
		JLabel lab1 = new JLabel("Import Requirements");
		Util.setFont(lab1, true, Util.FONT_MEDIUM);
		lab1.setHorizontalAlignment(SwingConstants.CENTER);
		northPanel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));
		northPanel.add(lab1);
		contentPane.add(northPanel, BorderLayout.NORTH);

		/* Centerpanel */
		centerPanel = new JPanel();
		contentPane.add(centerPanel, BorderLayout.CENTER);
		centerPanel.setLayout(new BorderLayout());

		/* Table */
		JPanel tablePanel = new JPanel();
		Object[] columnNames = { "Req. #", "Requirement", "Release #" };
		Object[][] data = createEmptyData(60);
		RequirementTableModel reqTableModel = new RequirementTableModel(data,
				columnNames);
		
		/* Create the table and set colors and tooltip texts */
		reqTable = new JTable(reqTableModel) {
	        public Component prepareRenderer(TableCellRenderer renderer,
	                                         int rowIndex, int vColIndex) {
	            Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
	            if (c instanceof JComponent) {
	                JComponent jc = (JComponent)c;
	                jc.setBackground(reqTable.getSelectedRow() == rowIndex ? 
	                        SystemColor.activeCaption : Color.WHITE);
	                
	                jc.setBackground(markedRows[rowIndex] ? 
	                        Color.RED : jc.getBackground());
	                
	                jc.setForeground(reqTable.getSelectedRow() == rowIndex ? 
	                        SystemColor.textHighlightText : SystemColor.textText);
	                
	                jc.setForeground(markedRows[rowIndex] ? 
	                        SystemColor.textText : jc.getForeground());
	                
	                if(getValueAt(rowIndex, vColIndex) != null &&
	                        ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
	                    jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
	                }
	                else {
	                    jc.setToolTipText(null);
	                }
	            }
	            return c;
	        }
			    protected void processKeyEvent(KeyEvent e) {
			        if(e.getID()==KeyEvent.KEY_PRESSED && e.getKeyCode() == KeyEvent.VK_ENTER) {
			            return;
			        }
			        else if(e.getID()==KeyEvent.KEY_RELEASED && e.getKeyCode() == KeyEvent.VK_ENTER) {
			            kh.keyTyped(new KeyEvent(reqTable, 0, 0, 0, KeyEvent.VK_ENTER, (char) KeyEvent.VK_ENTER));
			            return;
			        }
			        super.processKeyEvent(e);
			    }
	    };
	    
	    markedRows = new boolean[reqTable.getRowCount()];
	    
		reqTable.getColumnModel().getColumn(0).setMinWidth(50);
		reqTable.getColumnModel().getColumn(0).setPreferredWidth(100);	
		reqTable.getColumnModel().getColumn(1).setMinWidth(50);
		reqTable.getColumnModel().getColumn(1).setPreferredWidth(400);
		reqTable.getColumnModel().getColumn(2).setMinWidth(50);
		reqTable.getColumnModel().getColumn(2).setPreferredWidth(100);
		
		JScrollPane scrollPane = new JScrollPane(reqTable);
		tablePanel.setLayout(new BorderLayout());
		tablePanel.add(reqTable.getTableHeader(), BorderLayout.PAGE_START);
		tablePanel.add(scrollPane, BorderLayout.CENTER);
		centerPanel.add(tablePanel, BorderLayout.CENTER);
		
		reqTable.setEnabled(true);
		reqTable.getTableHeader().setReorderingAllowed(false);
		reqTable.setRowSelectionAllowed(true);
		reqTable.addMouseListener(mh);
		
		JPanel centerSouthPanel = new JPanel();
		centerSouthPanel.setLayout(new BoxLayout(centerSouthPanel, BoxLayout.X_AXIS));
		centerPanel.add(centerSouthPanel, BorderLayout.SOUTH);
		openButton = new JButton("Open");
		openButton.setToolTipText("Open file");
		Util.setJComponentSize(openButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		openButton.addActionListener(ah);
		acceptButton = new JButton("Accept");
		acceptButton.setToolTipText("Accept imported requirements");
		Util.setJComponentSize(acceptButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		acceptButton.addActionListener(ah);
		editRowButton = new JButton("Edit row");
		editRowButton.setToolTipText("Edit selected requirement");
		Util.setJComponentSize(editRowButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		editRowButton.addActionListener(ah);
		cancelButton = new JButton("Cancel");
		cancelButton.setToolTipText("Cancel import");
		Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		cancelButton.addActionListener(ah);

		
		centerSouthPanel.add(openButton);
		centerSouthPanel.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		centerSouthPanel.add(editRowButton);
		
		
		centerSouthPanel.add(Box.createHorizontalGlue());
		centerSouthPanel.add(acceptButton);
		centerSouthPanel.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		centerSouthPanel.add(cancelButton);

		centerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 20, 10));
		centerSouthPanel.setBorder(BorderFactory.createEmptyBorder(20, 0, 0, 0));

		
		/* Set up the menubar */
		JMenuBar menuBar;
		
		menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		JMenu editMenu = new JMenu("Edit");
		JMenu helpMenu = new JMenu("Help");
		
		openMenuItem = new JMenuItem("Open");
		openMenuItem.addActionListener(ah);
		acceptMenuItem = new JMenuItem("Accept");
		acceptMenuItem.addActionListener(ah);
		cancelMenuItem = new JMenuItem("Cancel");
		cancelMenuItem.addActionListener(ah);
		
		editRowMenuItem = new JMenuItem("Edit row...");
		editRowMenuItem.addActionListener(ah);
		clearTableMenuItem = new JMenuItem("Clear table");
		clearTableMenuItem.addActionListener(ah);
		
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ah);
		
		menuBar.add(fileMenu);
		fileMenu.add(openMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(acceptMenuItem);
		fileMenu.add(cancelMenuItem);
		
		menuBar.add(editMenu);
		editMenu.add(editRowMenuItem);
		editMenu.add(clearTableMenuItem);
		
		menuBar.add(helpMenu);
		helpMenu.add(helpMenuItem);
		
		
		this.setJMenuBar(menuBar);
	}
	
	/**
	 * Update the values in the table.
	 * 
	 * @param rList the list containing the Requirements
	 */
	public void updateReqTable(RequirementList rList) {
	    for(int k=0;k<reqTable.getRowCount();k++) {
	        reqTable.getModel().setValueAt("", k, 0);
			reqTable.getModel().setValueAt("", k, 1);
			reqTable.getModel().setValueAt("", k, 2);
	    }
		String[][] reqs = rList.getReqList(false); 
		for(int i=0;i<reqs.length;i++) {
			reqTable.getModel().setValueAt(reqs[i][0], i, 0);
			reqTable.getModel().setValueAt(reqs[i][1], i, 1);
			reqTable.getModel().setValueAt(reqs[i][2], i, 2);
		}
		this.getContentPane().requestFocusInWindow();
	}

	/**
	 * Accepts changes in a cell without having to press enter
	 */
	private void acceptEdit() {
		if (reqTable.getCellEditor() != null) {
			reqTable.getCellEditor().stopCellEditing();
		}
	}
	
	/**
	 * Method to create empty Table-data matrix
	 * 
	 * @param nbrOfRows number of rows to be created in the table
	 * @return the data matrix
	 */
	private Object[][] createEmptyData(int nbrOfRows) {
		Object[][] data = new Object[nbrOfRows][3];
		for(int k=0;k<nbrOfRows;k++) {
			for(int n=0;n<3;n++) {
				data[k][n] = "";
			}
		}
		return data;
	}
	
	public void setVisible(boolean visible) {
	    reqTable.requestFocus();
	    reqTable.setRowSelectionInterval(0, 0);
	    super.setVisible(visible);
	}
	
	/**
     * ActionListener to handle user interaction.
     * 
     * @author Mikael Jnsson & Per Klingns
     */
	class ActionHandler implements ActionListener {
	    
	    private ImportWindow iw;
	    
	    public ActionHandler(ImportWindow iw) {
	        this.iw = iw;
	    }

		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == acceptButton || e.getSource() == acceptMenuItem) {
			    iw.acceptEdit();
			    boolean tableValid = true;
			    boolean duplicateReqNbrs = false;
			    int nbrOfReqs = 0;
			    for(int i=0;i<reqTable.getRowCount();i++) {
			        if(((String) reqTable.getValueAt(i,1)).length() > 0 &&
			                ((String) reqTable.getValueAt(i,0)).length() < 1) {
			            tableValid = false;
			            reqTable.setRowSelectionInterval(i,i);
			            markedRows[i] = true;
			        }
			        else if(((String) reqTable.getValueAt(i,0)).length() > 0) {
			            nbrOfReqs++;
			            markedRows[i] = false;
			        }
			        else {
			            markedRows[i] = false;
			        }
			        for(int k=0;k<i;k++) {
			            if(((String) reqTable.getValueAt(i,0)).length() > 0 
			                    && ((String) reqTable.getValueAt(i,0)).
			                    compareTo(((String) reqTable.getValueAt(k,0)))==0) {
			                duplicateReqNbrs = true;
			                markedRows[i] = true;
			                markedRows[k] = true;
			            }
			        }
			        
			    }
			    /* 
			     * Display an error message if there are duplicate requirement
			     * numbers.
			     */
			    if(duplicateReqNbrs) {
			        JOptionPane.showMessageDialog(null,
							"The requirement numbers must be unique",
							"Duplicate requirement numbers", JOptionPane.WARNING_MESSAGE);
			        reqTable.repaint();
			    }
			    /*
			     * Display an error message if requirement numbers are missing
			     */
			    else if(!tableValid) {
			        JOptionPane.showMessageDialog(null,
							"One or more requirement numbers are missing",
							"Missing requirement number(s)", JOptionPane.WARNING_MESSAGE);
			        reqTable.repaint();
			    }
			    /*
			     * Display an error message if not enough requirements are imported
			     */
			    else if(nbrOfReqs < 2) {
			        JOptionPane.showMessageDialog(null,
							"At least two requirements must be imported",
							"Too few requirements", JOptionPane.WARNING_MESSAGE);
			        reqTable.repaint();
			    }
			    /* Everything is OK */
			    else {
			        p.setImportedList(((DefaultTableModel) reqTable.getModel())
			                .getDataVector());
			        dispose();
			    }
			} 
			else if (e.getSource() == editRowButton || e.getSource()==editRowMenuItem) {
				ew.editRow(reqTable.getSelectedRow());
				iw.setEnabled(false);	
			} 
			else if (e.getSource() == cancelButton || e.getSource() == cancelMenuItem) {
				dispose();
			}
			else if(e.getSource() == openButton || e.getSource() == openMenuItem) {
			    JFileChooser chooser = new JFileChooser();
				chooser.addChoosableFileFilter(new Filter("xls"));
				int returnVal = chooser.showOpenDialog(null);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File file = chooser.getSelectedFile();
                    if(file.getAbsolutePath().endsWith(".xls")) {
                        new ExcelImporter(iw, file, p);
                    }
                    else {
                        JOptionPane.showMessageDialog(null, "Unsupported file type",
            					"Failed", JOptionPane.ERROR_MESSAGE);
                    }
                } 
			}
			
			else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("importwindow.html");
			}
			
			else if(e.getSource() == clearTableMenuItem) {
			    int response = JOptionPane.showConfirmDialog (null,
                        "This will remove all information in the table.\n" +
                        "Do you want to continue?",
                        "Confirm clearance",
                        JOptionPane.OK_CANCEL_OPTION,
                        JOptionPane.QUESTION_MESSAGE);
			    if (response == JOptionPane.OK_OPTION) {
			        for(int r=0;r<reqTable.getRowCount();r++) {
						for(int c=0;c<3;c++) {
							reqTable.setValueAt("", r, c);
						}
					}
			    }
			}
			/* Buttons pressed in EditWindow */
			else if(e.getSource() == ew.acceptButton) {
				ew.acceptEdit();
				reqTable.setValueAt(ew.editTable.getValueAt(0,0), ew.row, 0);
				reqTable.setValueAt(ew.editTable.getValueAt(0,1), ew.row, 1);
				reqTable.setValueAt(ew.editTable.getValueAt(0,2), ew.row, 2);
				reqTable.setRowSelectionInterval(reqTable.getSelectedRow()+1, reqTable.getSelectedRow()+1);
				ew.cancelEdit();
				ew.setVisible(false);
				iw.setEnabled(true);
				iw.toFront();
			} 
			else if(e.getSource() == ew.cancelButton) {
				ew.cancelEdit();
				ew.setVisible(false);
				iw.setEnabled(true);
				iw.toFront();
			} 
			
			
		}
	}

	/**
	 * Defines a window for editing the imported requirements
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class EditWindow extends JFrame {
		private Container contentPane;
		private JTable editTable;
		private JButton acceptButton, cancelButton;
		private int row;
		
		/**
		 * constructs an EditWindow
		 */
		private EditWindow() {
			super("Edit requirement");
			init();
		}
		
		/**
		 * Initializes the graphical user interface
		 */
		private void init() {
			this.setSize(600,200);
			this.setResizable(false);
			setIconImage(getToolkit().getImage("images\\rainbow.gif"));
			contentPane = this.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.addKeyListener(kh);
			this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			
			/* North panel */
			JPanel north = new JPanel();
			north.setBorder(BorderFactory.createEmptyBorder(10,0,5,0));
			JLabel headerLabel = new JLabel("Edit selected requirement");
			Util.setFont(headerLabel, true, Util.FONT_SMALL);
			headerLabel.setHorizontalAlignment(SwingConstants.CENTER);
			north.add(headerLabel);
			contentPane.add(north, BorderLayout.NORTH);
			
			/* Center panel */
			Object[] columnNames = { "Req. #", "Requirement", "Release #" };
			Object[][] data = createEmptyData(1);
			DefaultTableModel editTableModel = new DefaultTableModel(data, columnNames);
			editTable = new JTable(editTableModel) {
			    protected void processKeyEvent(KeyEvent e) {
			        if(e.getID()==KeyEvent.KEY_RELEASED && e.getKeyCode() == KeyEvent.VK_ENTER) {
			            kh.keyTyped(new KeyEvent(editTable, 0, 0, 0, KeyEvent.VK_ENTER, (char) KeyEvent.VK_ENTER));
			            return;
			        }
			        else if(e.getID()==KeyEvent.KEY_RELEASED && e.getKeyCode()==KeyEvent.VK_ESCAPE) {
			            kh.keyTyped(new KeyEvent(editTable, 0, 0, 0, KeyEvent.VK_ESCAPE, (char) KeyEvent.VK_ESCAPE));
			            return;
			        }
			        super.processKeyEvent(e);
			    }
			};
			editTable.getColumnModel().getColumn(0).setPreferredWidth(100);
			editTable.getColumnModel().getColumn(0).setMinWidth(50);
			editTable.getColumnModel().getColumn(1).setPreferredWidth(400);
			editTable.getColumnModel().getColumn(1).setMinWidth(50);
			editTable.getColumnModel().getColumn(2).setPreferredWidth(100);
			editTable.getColumnModel().getColumn(2).setMinWidth(50);
			editTable.addKeyListener(kh);
					
			JPanel center = new JPanel();
			contentPane.add(center, BorderLayout.CENTER);
			center.setLayout(new BorderLayout());
			center.add(editTable.getTableHeader(), BorderLayout.PAGE_START);
			center.add(editTable, BorderLayout.CENTER);
			center.setBorder(BorderFactory.createEmptyBorder(5,10,15,10));
			
			/* South panel */
			acceptButton = new JButton("Accept");
			Util.setJComponentSize(acceptButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
			acceptButton.addActionListener(ah);
			cancelButton = new JButton("Cancel");
			Util.setJComponentSize(cancelButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
			cancelButton.addActionListener(ah);
			JPanel south = new JPanel();
			south.setLayout(new BoxLayout(south, BoxLayout.X_AXIS));
			
			south.add(Box.createHorizontalGlue());
			south.add(acceptButton);
			south.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
			south.add(cancelButton);
			south.add(Box.createHorizontalGlue());
			
			south.setBorder(BorderFactory.createEmptyBorder(5,0,10,0));
			contentPane.add(south, BorderLayout.SOUTH);
			
			pack();
			Util.centerJFrame(this);
			setVisible(false);
		}

		/**
		 * Accepts changes in a cell without having to press enter
		 */
		private void acceptEdit() {
			if (editTable.getCellEditor() != null) {
				editTable.getCellEditor().stopCellEditing();
			}
		}
		
		/**
		 * Correctly cancels changes in the EditWindow
		 */
		private void cancelEdit() {
			editTable.setValueAt("",0,0);
			editTable.setValueAt("",0,1);
			editTable.setValueAt("",0,2);
			if (editTable.getCellEditor() != null) {
		        editTable.getCellEditor().cancelCellEditing();
		    }
		}
			
		/**
		 * Sets the values in the EditWindow to the values on the 
		 * opened row and opens the EditWindow.
		 * 
		 * @param row the index of the opened row in ImportWindow
		 */
		private void editRow(int row) {

			String reqNbr = ""; // requirement number
			String req = "";    // requirement description
			String relNbr = ""; // release number
			if(reqTable.getValueAt(row,0) != null) {
			    reqNbr = reqTable.getValueAt(row,0).toString();
			}
			if(reqTable.getValueAt(row,1) != null) {
			    req = reqTable.getValueAt(row,1).toString();
			}
			if(reqTable.getValueAt(row,2) != null) {
			    relNbr = reqTable.getValueAt(row,2).toString();
			}	
			editTable.setValueAt(reqNbr,0,0);
			editTable.setValueAt(req,0,1);
			editTable.setValueAt(relNbr,0,2);
			this.row = row;
			editTable.editCellAt(0, 0);
			editTable.requestFocus();
			editTable.changeSelection(0, 0, false, false);
			setVisible(true);
		}
		
		public void dispose() {
		    ah.actionPerformed(new ActionEvent(cancelButton, 0, null));
		}
	}
	
	/**
	 * KeyListener that handles keypresses.
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class KeyHandler implements KeyListener {
		public void keyPressed(KeyEvent e) {
			if(e.getKeyCode()==KeyEvent.VK_UP && reqTable.getSelectedRow()!=0
					&& e.getSource() != ew) {
				reqTable.setRowSelectionInterval(reqTable.getSelectedRow()-1, 
						reqTable.getSelectedRow()-1);
			}
			else if(e.getKeyCode()==KeyEvent.VK_DOWN && e.getSource() != ew
					&& reqTable.getSelectedRow()!=reqTable.getRowCount()-1) {
				reqTable.setRowSelectionInterval(reqTable.getSelectedRow()+1, 
						reqTable.getSelectedRow()+1);
			}
			else if(e.getKeyCode()==KeyEvent.VK_ESCAPE) {
				ew.cancelEdit();
				ew.setVisible(false);
			}
			
		}
		public void keyReleased(KeyEvent e) { }
		public void keyTyped(KeyEvent e) { 
		    if(e.getSource()==ew.editTable && e.getKeyCode()==KeyEvent.VK_ENTER) {
			    ah.actionPerformed(new ActionEvent(ew.acceptButton, 0, null));
			}
			else if(e.getSource()==reqTable && e.getKeyCode()==KeyEvent.VK_ENTER) {
			    ah.actionPerformed(new ActionEvent(editRowButton, 0, null));
			}
			else if(e.getSource()==ew.editTable && e.getKeyCode()==KeyEvent.VK_ESCAPE) {
			    ah.actionPerformed(new ActionEvent(ew.cancelButton, 0, null));
			}
		}
	}
	
	/**
	 * MouseListener to handle user interaction
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class MouseHandler implements MouseListener {
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount()==1) {
				reqTable.setRowSelectionInterval(
						reqTable.rowAtPoint(e.getPoint()),
						reqTable.rowAtPoint(e.getPoint()));
			} 
			else if(e.getClickCount()==2) {
				ah.actionPerformed(new ActionEvent(editRowButton, 0, ""));
			}
		}		
		public void mouseEntered(MouseEvent e){ }
		public void mouseExited(MouseEvent e) { }
		public void mousePressed(MouseEvent e) { }
		public void mouseReleased(MouseEvent e) { }
	}
	
	/**
	 * DefaultTableModel used to define desired behaviour of the table
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class RequirementTableModel extends DefaultTableModel {
		
		public RequirementTableModel(Object[][] data, Object[] columnNames) {
			super(data, columnNames);
		}
		
		/* 
		 * Make sure that only the last column in ImportWindow
		 * is editable. 
		 */
		public boolean isCellEditable(int row, int column) {
			if (column == 2) {
				return true;
			}
			return false;
		}
	}
}

