/*
 * ImprovementsWindow.java
 *
 * Created on 2005-feb-08
 */

package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;


import rainbowie.*;
import util.ExcelWriter;
import util.Filter;
import util.Util;

/**
 * Defines an ImprovementWindow where root-causes and improvement suggestions 
 * are compiled.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ImprovementsWindow extends JFrame {
	private AdjustmentHandler adjh;
	private ActionHandler ah;
	private MouseHandler mh;
	
	private JPanel center, south, north;
	
	private Container contentPane;
	private GraphWindow gw;
	
	private JButton exportButton, closeButton, removeButton, addRowButton;
	private JMenuItem exportMenuItem, closeMenuItem, addRowMenuItem, removeMenuItem, 
	helpMenuItem;
	
	private Rainbowie p;
	private DefaultTableModel requirementsTableModel;
	private JScrollPane rootCauseScrollPane, requirementsScrollPane;	
	private JTable rootCauseTable, requirementsTable;
	private RootCauseTableModel rootCauseTableModel;
	
	private ReqTableCellRenderer rtcr;
	
	/**
	 * Constructs an ImprovementWindow
	 * 
	 * @param p reference to the programs Parseq object
	 * @param gw reference to the GraphWindow
	 * @param windowNbr Window identifier that shows which graph this 
	 * window belong to
	 */
	public ImprovementsWindow(Rainbowie p, GraphWindow gw, int windowNbr) {
			super("Root-Cause Matrix #" + windowNbr);
			this.p = p;			
			this.gw = gw;
			rtcr = new ReqTableCellRenderer();
			init();
	}
	
	/**
	 * Initializes the graphical user interface
	 */
	private void init() {

		contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout());
		ah = new ActionHandler();
		adjh = new AdjustmentHandler();
		mh = new MouseHandler();

		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setSize(850,500);
		this.setResizable(true);
		setIconImage(getToolkit().getImage("images\\rainbow.gif"));
		
		/* North */
		north = new JPanel();
		JLabel northLabel = new JLabel("Root-Cause Matrix");
		Util.setFont(northLabel, true, Util.FONT_MEDIUM);
		north.add(northLabel);
		contentPane.add(north, BorderLayout.NORTH);
		
		/* Center */
		center = new JPanel();
		center.setBorder(BorderFactory.createEmptyBorder(0,20,0,20));
		center.setLayout(new BoxLayout(center, BoxLayout.X_AXIS));	
		String[] colNames = {"Root Causes","Improvements", "Export" };
		Object[][] data = new Object[60][3];
		for (int i=0;i<60;i++) {
		    data[i][0] = "";
		    data[i][1] = "";
		    data[i][2] = new Boolean(false);		    
		}

		rootCauseTableModel = new RootCauseTableModel(data, colNames);

		/* Create the table and set tooltip texts */
		rootCauseTable = new JTable(rootCauseTableModel) {
	        public Component prepareRenderer(TableCellRenderer renderer,
	                                         int rowIndex, int vColIndex) {
	            Component c = super.prepareRenderer(renderer, rowIndex, vColIndex);
	            if (c instanceof JComponent) {
	                JComponent jc = (JComponent)c;
	                if(vColIndex < 2 && getValueAt(rowIndex, vColIndex) != null &&
	                        ((String) getValueAt(rowIndex, vColIndex)).length()>0) {
	                    jc.setToolTipText((String)getValueAt(rowIndex, vColIndex));
	                }
	                else {
	                    jc.setToolTipText(null);
	                }
	            }
	            return c;
	        }
	    };
		rootCauseTable.getTableHeader().setReorderingAllowed( false );
		
		rootCauseScrollPane = new JScrollPane(rootCauseTable);
		rootCauseScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		rootCauseScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		rootCauseScrollPane.setAlignmentY(Component.TOP_ALIGNMENT);
		setColumnWidth(rootCauseTable, true);
		rootCauseScrollPane.getVerticalScrollBar().addAdjustmentListener(adjh);
		
		requirementsTableModel = new RequirementsTableModel();
		requirementsTable = new JTable(requirementsTableModel);
		
		requirementsTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		requirementsTable.setRowSelectionAllowed(false);
		requirementsTable.setColumnSelectionAllowed(true);
		requirementsTable.setCellSelectionEnabled(false);
		requirementsTable.getTableHeader().setReorderingAllowed(false);
		requirementsTable.setEnabled(false);
		requirementsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION );
		requirementsTable.addMouseListener(mh);
		requirementsTable.getTableHeader().addMouseListener(mh);
		
		requirementsScrollPane = new JScrollPane(requirementsTable);
		requirementsScrollPane.setVerticalScrollBarPolicy(
		        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		requirementsScrollPane.setHorizontalScrollBarPolicy(
		        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		Util.setJComponentSize(requirementsScrollPane, 
		        (requirementsTable.getColumnCount())*50+18, 400);
		requirementsScrollPane.setAlignmentY(Component.TOP_ALIGNMENT);
		requirementsScrollPane.getVerticalScrollBar().addAdjustmentListener(adjh);
		
		center.add(Box.createGlue());
		center.add(rootCauseScrollPane);
		center.add(Box.createHorizontalStrut(-3));
		center.add(requirementsScrollPane);
		center.add(Box.createGlue());
		
		contentPane.add(center, BorderLayout.CENTER);
		
		/* South */
		south = new JPanel();
		south.setLayout(new BoxLayout(south, BoxLayout.X_AXIS));
		exportButton = new JButton("Export");
		closeButton = new JButton("Close");
		removeButton = new JButton("Remove");
		removeButton.setEnabled(false);
		addRowButton = new JButton("Add row");
		
		Util.setJComponentSize(exportButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(closeButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(removeButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		Util.setJComponentSize(addRowButton, Util.BUTTON_WIDTH, Util.BUTTON_HEIGHT);
		
		exportButton.setToolTipText("Export table to an Excel file");
		closeButton.setToolTipText("Close this window");
		removeButton.setToolTipText("Remove the selected column/requirement");
		addRowButton.setToolTipText("Add a new row at the end of the table");
		
		addRowButton.addActionListener(ah);
		exportButton.addActionListener(ah);
		closeButton.addActionListener(ah);
		removeButton.addActionListener(ah);
		
		south.add(Box.createHorizontalGlue());
		south.add(exportButton);
		south.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		south.add(addRowButton);
		south.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		south.add(removeButton);
		south.add(Box.createHorizontalStrut(Util.HORIZONTAL_BUTTON_DISTANCE));
		south.add(closeButton);
		south.add(Box.createHorizontalGlue());
		south.setBorder(BorderFactory.createEmptyBorder(20, 0, 20, 0));
		contentPane.add(south, BorderLayout.SOUTH);
		repaint();
		
		/* Menu */
		JMenuBar menuBar;
		JMenu fileMenu, editMenu, helpMenu;
		
		menuBar = new JMenuBar();
		
		fileMenu = new JMenu("File");
		exportMenuItem = new JMenuItem("Export");
		exportMenuItem.addActionListener(ah);
		closeMenuItem = new JMenuItem("Close");
		closeMenuItem.addActionListener(ah);
		
		fileMenu.add(exportMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(closeMenuItem);
		menuBar.add(fileMenu);
		
		editMenu = new JMenu("Edit");
		addRowMenuItem = new JMenuItem("Add Row");
		removeMenuItem = new JMenuItem("Remove Column");
		removeMenuItem.setEnabled(false);
		addRowMenuItem.addActionListener(ah);
		removeMenuItem.addActionListener(ah);
		editMenu.add(addRowMenuItem);
		editMenu.add(removeMenuItem);
		menuBar.add(editMenu);
		
		helpMenu = new JMenu("Help");
		helpMenuItem = new JMenuItem("How To");
		helpMenuItem.addActionListener(ah);
		helpMenu.add(helpMenuItem);
		menuBar.add(helpMenu);
		
		this.setJMenuBar(menuBar);
		Util.centerJFrame(this);
		setVisible(true);
	}
	
	/**
	 * Accepts changes in a cell without having to press enter
	 */
	private void acceptEdit() {
		if (rootCauseTable.getCellEditor() != null) {
		    rootCauseTable.getCellEditor().stopCellEditing();
		}
	}
	
	/**
	 * Sets the column widths for the current table
	 * 
	 * @param table the table which shall be adusted
	 * @param rootCause true if the current table is the 
	 * root-cause table (left part), false if the current 
	 * table is the requirements table (right part).
	 */
	private void setColumnWidth(JTable table, boolean rootCause) {
		if (rootCause) {
			table.getColumnModel().getColumn(0).setMinWidth(100);
			table.getColumnModel().getColumn(1).setMinWidth(100);
			table.getColumnModel().getColumn(2).setMaxWidth(75);
			table.getColumnModel().getColumn(2).setPreferredWidth(50);
			table.getColumnModel().getColumn(2).setMinWidth(50);
		}
		else {
			for(int i=0;i<table.getColumnCount();i++) {
				table.getColumnModel().getColumn(i).setPreferredWidth(60);
				table.getColumnModel().getColumn(i).setMinWidth(50);
				table.getColumnModel().getColumn(i).setMaxWidth(150);
			}
		}
	}
	
	/**
	 * Set the sizes of the scrollpanes due to the number of 
	 * added requirement columns
	 */
	private void setWindowSize() {
	    int widthFactor = (requirementsTable.getColumnCount() < 5 ? 
	            requirementsTable.getColumnCount() : 5);
	    Util.setJComponentSize(rootCauseScrollPane, 
				center.getWidth()-60*widthFactor-53, center.getHeight()-15);
	    Util.setJComponentSize(requirementsScrollPane, 
				60*widthFactor+20, center.getHeight());
	    rootCauseTable.invalidate();
	}
	
	/**
	 * Add a requirement column to the requirement table 
	 * (right part of the table)
	 * 
	 * @param r the requirement to add
	 */
	public void addReq(Requirement r) {
		int rows = 60;
		if(requirementsTable.getRowCount()!=0) {
			rows = requirementsTable.getRowCount();
		}
		if(r!=null) {
		    requirementsTableModel.addColumn(r.getReqNbr(), new Object[rows]);
		}
		
		javax.swing.table.TableColumn tc;
		for(int i=0; i<requirementsTable.getColumnCount(); i++){
		    tc = requirementsTable.getColumnModel().getColumn(i);
		    tc.setCellRenderer(rtcr);
		}
		
		setColumnWidth(rootCauseTable, true);
		setColumnWidth(requirementsTable, false);
		setVisible(true);
	}
	
	/**
	 * Makes sure that the window size and column widths are
	 * adjusted before the super-class call to 
	 * JFrame.paint(Graphics g) is done
	 */
	public void paint(Graphics g) {
		setWindowSize();
		setColumnWidth(rootCauseTable, true);
		
		this.invalidate();
		this.validate();

		super.paint(g);
	}
	
	
	/**
	 * Removes the specified column from the table and the associated
	 * data from the table model
	 * 
	 * @param table the table where the column shall be removed
	 * @param vColIndex the index of the column to remove
	 */
    public void removeColumnAndData(JTable table, int vColIndex) {
        RequirementsTableModel model = (RequirementsTableModel)table.getModel();
        TableColumn col = table.getColumnModel().getColumn(vColIndex);
        int columnModelIndex = col.getModelIndex();
        Vector data = model.getDataVector();
        Vector colIds = model.getColumnIdentifiers();
    
        /* Remove the column from the table */
        table.removeColumn(col);
    
        /* Remove the column header from the table model */
        colIds.removeElementAt(columnModelIndex);
    
        /* Remove the column data */
        for (int r=0; r<data.size(); r++) {
            Vector row = (Vector)data.get(r);
            row.removeElementAt(columnModelIndex);
        }
        model.setDataVector(data, colIds);
        
        /*
         * Correct the model indices in the TableColumn objects by
         * decrementing those indices that follow the deleted column
         */
        Enumeration enum = table.getColumnModel().getColumns();
        for (; enum.hasMoreElements(); ) {
            TableColumn c = (TableColumn)enum.nextElement();
            if (c.getModelIndex() >= columnModelIndex) {
                c.setModelIndex(c.getModelIndex()-1);
            }
        }
		
        model.fireTableStructureChanged();
        setColumnWidth(rootCauseTable, true);
		setColumnWidth(requirementsTable, false);
		repaint();
    }

	
	/**
	 * ActionListener to handle user interaction.
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class ActionHandler implements ActionListener {
		
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == addRowButton || e.getSource() == addRowMenuItem) {
			    Object[] row = {"", "", new Boolean(false)};
				rootCauseTableModel.addRow(row);
				requirementsTableModel.addRow(new Object[requirementsTable.getColumnCount()]);
			}
			/* Export the matrix (and graph) to an Excel spreadsheet */
			else if (e.getSource() == exportButton || e.getSource() == exportMenuItem) {
			    acceptEdit();
			    JFileChooser chooser = new JFileChooser();
			    chooser.addChoosableFileFilter(new Filter("xls"));
			    int returnVal = chooser.showSaveDialog(null);
			    if (returnVal == JFileChooser.APPROVE_OPTION) {
			        
			        File oldFile = chooser.getSelectedFile();
			        int response = JOptionPane.OK_OPTION;
			        if(oldFile.exists()) {
			            response = JOptionPane.showConfirmDialog (null,
			                    "Overwrite existing file?","Confirm Overwrite",
			                    JOptionPane.OK_CANCEL_OPTION,
			                    JOptionPane.QUESTION_MESSAGE);
			            
			        } 
			        if (response == JOptionPane.OK_OPTION) {
			            File newFile;
			            if(!oldFile.getName().endsWith(".xls")) {
			                newFile = new File(oldFile.getAbsolutePath()+".xls");
			            } else {
			                newFile = oldFile;
			            }
			            
			            ExcelWriter ew = new ExcelWriter(newFile);
			            ew.writeTable(rootCauseTable, requirementsTable, "Root Cause Table");
			            
			            int n = JOptionPane.showConfirmDialog(null, 
			                    "Do you want to include the graph?",
			                    "Graph option", JOptionPane.YES_NO_OPTION);
			            if(n == JOptionPane.YES_OPTION) {
			                ew.writeGraph(gw.getGraphImage(), "Requirements graph");
			            }
			            
			            ew.writeAndCloseWorkbook();
			        }
				}
			}
			/* Remove a column from the requirement table */
			else if(e.getSource() == removeButton || e.getSource() == removeMenuItem) {
			    removeButton.setEnabled(false);
			    removeMenuItem.setEnabled(false);
			    removeColumnAndData(requirementsTable, rtcr.getSelectedColumn());
			    javax.swing.table.TableColumn tc;
				for(int i=0; i<requirementsTable.getColumnCount(); i++){
				    tc = requirementsTable.getColumnModel().getColumn(i);
				    tc.setCellRenderer(rtcr);
				}
				setColumnWidth(requirementsTable, false);
			    rtcr.setSelectedColumn(-1);
			    requirementsTable.repaint();
			}
			else if(e.getSource() == closeButton || e.getSource() == closeMenuItem) {
			    dispose();
			}
			else if (e.getSource() == helpMenuItem) {
			    p.showHelpDialog("improvementswindow.html");
			}
		}
	}
	
	/**
	 * Synchronizes scrolling of the right table with scrolling 
	 * in the left table
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class AdjustmentHandler implements AdjustmentListener {

		public void adjustmentValueChanged(AdjustmentEvent e) {
			if (e.getSource() == requirementsScrollPane.getVerticalScrollBar()) {
				rootCauseScrollPane.getVerticalScrollBar().setValue(requirementsScrollPane.getVerticalScrollBar().getValue());
			} else if (e.getSource() == rootCauseScrollPane.getVerticalScrollBar()) {
				requirementsScrollPane.getVerticalScrollBar().setValue(rootCauseScrollPane.getVerticalScrollBar().getValue());
			}

		}	
	}
	
	
	/**
	 * MouseListener to handle user interaction
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class MouseHandler implements MouseListener {
	       
	    public void mouseClicked(MouseEvent e) {
	        if (e.getSource() == requirementsTable) {
	            if(e.getClickCount()==2) {
	                int row = requirementsTable.rowAtPoint(e.getPoint());
	                int col = requirementsTable.columnAtPoint(e.getPoint());
	                rootCauseTable.setRowSelectionInterval(row, row);
	                
	                if ( ((String)requirementsTable.getModel().getValueAt(row, col)) == null ) {
	                    requirementsTable.getModel().setValueAt("X", row, col);
	                    requirementsTable.getColumnModel().getColumn(col).getCellRenderer().getTableCellRendererComponent(requirementsTable, "X", false, false, row, col);
	                } else {
	                    requirementsTable.getModel().setValueAt(null, row, col);
	                    requirementsTable.getColumnModel().getColumn(col).getCellRenderer().getTableCellRendererComponent(requirementsTable, "", false, false, row, col);
	                }
	            }
	        }
	        else if(e.getSource() == requirementsTable.getTableHeader()) {
	            if(e.getClickCount()==1) {
	                rtcr.setSelectedColumn(requirementsTable.columnAtPoint(e.getPoint()));
	                requirementsTable.repaint();
	            }
	        }
	    }
        public void mouseEntered(MouseEvent e) { }
        public void mouseExited(MouseEvent e) { }
        public void mousePressed(MouseEvent e) { }
        public void mouseReleased(MouseEvent e) { }
	}
	
	/**
	 * DefaultTableModel used to define desired 
	 * behaviour of the root cause table
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class RootCauseTableModel extends DefaultTableModel { 
	        
	    public RootCauseTableModel(Object[][] data, String[] columnNames) {
	        super(data, columnNames);

	    }
    
	    /** 
	     * JTable uses this method to determine the default renderer
	     * editor for each cell. If we didn't implement this method, 
	     * then the last column would contain text ("true"/"false"), 
	     * rather than a check box. 
	     */ 
	    public Class getColumnClass(int c) { 
	        return getValueAt(0, c).getClass(); 
	    } 
	    
	    /**
	     * Defines which cells that are editable
	     */
	    public boolean isCellEditable(int row, int col) { 
	        if (col < 3) { 
	            return true; 
	        } 
	        return false; 
	    } 
	    
	    /**
	     * Set the value in the specified cell
	     * 
	     * @param value the value to set in the cell
	     * @param row 	the cells row index
	     * @param col	the cells column index
	     */
	    public void setValueAt(Object value, int row, int col) {
	        super.setValueAt(value, row, col);
	        fireTableCellUpdated(row, col); 
	    }    
	}
	
	/**
	 * DefaultTableModel used to define desired 
	 * behaviour of the root cause table
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class RequirementsTableModel extends DefaultTableModel {
	    
	    public Vector getColumnIdentifiers() {
            return columnIdentifiers;
        }
	}
	
	/**
	 * TableCellRenderer to set color marking and cell selection of 
	 * the requirement table
	 * 
	 * @author Mikael Jnsson & Per Klingns
	 */
	class ReqTableCellRenderer extends JLabel implements TableCellRenderer { 
	    
	    private int selectedCol = -1;
	    
	    public ReqTableCellRenderer(){ 
	        setOpaque(true); 
	    } 
	    
	    public Component getTableCellRendererComponent(JTable table, Object value, 
	            boolean isSelected, boolean hasFocus, int row, int column) { 
	        setForeground(Color.BLACK); 
	        setHorizontalAlignment(CENTER);
	        if(table.getColumnCount() > 0) {
	            this.setToolTipText(p.getPrioList().getReq(
	                    table.getColumnName(column)).getReq());
	        }
	        
	        if(column == selectedCol) {
	            setBackground(SystemColor.activeCaption);
	            setForeground(SystemColor.textHighlightText);
	        }
	        else {
	            setBackground(new Color(240,240,240));
	            setForeground(SystemColor.textText);
	        }

	        if(value!=null) {
	            setText(value.toString());
	        }
	        else {
	            setText("");
	        }
	        acceptEdit();
	        return this;
	    }
	    
	    /* Keep track of the selected column */
	    public void setSelectedColumn(int selectedCol) {
	        if(selectedCol == -1) {
	            this.selectedCol = -1;
	        }
	        else if(this.selectedCol == -1 || selectedCol != this.selectedCol) {
	            this.selectedCol = selectedCol;
	            removeButton.setEnabled(true);
	            removeMenuItem.setEnabled(true);
	        }
	        else {
	            this.selectedCol = -1;
	            removeButton.setEnabled(false);
	            removeMenuItem.setEnabled(false);
	        }
	    }
	    
	    public int getSelectedColumn() {
	        return selectedCol;
	    }
	}
}
