/*
 * DollarTechnique.java
 *
 * Created on 2005-feb-23
 */

package prioritizer;

import rainbowie.RequirementList;

/**
 * Defines the $100 technique used.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class DollarTechnique {
    private String criteria;
    private int[] currentPrio;
    /* Keeps track of the current amount used */
    private int dollarsUsedInCurrent; 
    /* Keeps track of total amount if there is more than one user */
    private int dollarsUsedInTotal; 
    
    private RequirementList rList;
    private int[] totalPrio1, totalPrio2;
    private int dollarsAvailible;
    
    /**
     * Constructor for the DollarTechnique.
     * 
     * @param rList the RequirementList containing the Requirements to be 
     * prioritized 
     */
    public DollarTechnique(RequirementList rList) {
        this.rList = rList;
        currentPrio = new int[rList.getLength()];
        totalPrio1 = new int[rList.getLength()];
        totalPrio2 = new int[rList.getLength()];
        dollarsUsedInTotal = 0;
        dollarsAvailible = 100;
        criteria = rList.getPrioCriteria1();
    }
    
    /**
     * Returns the number of dollars availible in current prioritization
     *
     *	@return the number of dollars availible
     */
    public int getDollarsAvailible() {
        return dollarsAvailible;
    }
    
    /**
     * Sets the number of dollars availible in current prioritization
     * 
     * @param dollarsAvailible the number of dollars availible
     */
    public void setDollarsAvailible(int dollarsAvailible) {
        this.dollarsAvailible = dollarsAvailible;
    }
    
    /**
     * Sets the priorities for the current criteria.
     * <br><br>
     * Is called when a criteria has been completed in the normal
     * $100 technique. 
     */
    public void criteriaDone() {
        for(int i=0;i<currentPrio.length;i++) {
            if(criteria.compareTo(rList.getPrioCriteria1())==0) {
                totalPrio1[i] = totalPrio1[i] + currentPrio[i];
            }
            else {
                totalPrio2[i] = totalPrio2[i] + currentPrio[i];
            }
            currentPrio[i] = 0;
        }
        dollarsUsedInCurrent = 0;
    }
    
    /**
     * Assigns the priorities to the requirements in the RequirementList.
     */
    public void done() {
        criteriaDone();
        dollarsUsedInTotal += dollarsAvailible;      
        for(int i=0;i<totalPrio1.length;i++) {
            if(rList.getReq(i).getReqNbr().length()>0) {
                    rList.getReq(i).setPriority1(((dollarsAvailible*1.0)*totalPrio1[i])/dollarsUsedInTotal);
                    rList.getReq(i).setPriority2(((dollarsAvailible*1.0)*totalPrio2[i])/dollarsUsedInTotal);             
            }
        }
    }
    
    /**
     * Is only used when going to the $100 technique from the PlanningGame.
     * Sets priorities for the current criteria in the RequirementList.
     * 
     * @param criteria the current criteria
     */
    public void oneCriteriaDone(String criteria) {
        for(int i=0;i<currentPrio.length;i++) {
            if(rList.getReq(i).getReqNbr().length()>0) {
                if(criteria.compareTo(rList.getPrioCriteria1())==0) {
                    rList.getReq(i).setPriority1(1.0*currentPrio[i]);
                }
                else {
                    rList.getReq(i).setPriority2(1.0*currentPrio[i]);
                }
            }
        }
    }
    
    /**
     * Completes a criteria and prepares the program for the next user
     * to prioritize.
     */
    public void nextUser() {
        criteriaDone();
        dollarsUsedInTotal += dollarsAvailible;
        dollarsUsedInCurrent = 0;
    }
    
    /**
     * Sets the current criteria.
     * 
     * @param criteria the criteria.
     */
    public void setCriteria(String criteria) {
        this.criteria = criteria;
    }
    
    /**
     * Calculates how many dollar that have been used so far and checks if the 
     * new value will exceed the $100 limit or not.
     * 
     * @param value the new value to try and set
     * @param row   the row in DollarWindow and index in the priority array
     * @return      <li>The current amount of $ used if the $100 is not exceeded
     * 				<li>-1 if the $100 limit is exceeded
     */
    public int setValue(int value, int row) {
        if(dollarsAvailible-value-dollarsUsedInCurrent+currentPrio[row] >= 0) {
            dollarsUsedInCurrent += value - currentPrio[row];
            currentPrio[row] = value;
            
            return dollarsUsedInCurrent;
        }
        return -1; // $ limit exceeded
    }    
}
