/*
 * Pairwise.java
 *
 * Created on 2005-mar-18
 */

package prioritizer;

import java.util.LinkedList;
import rainbowie.Requirement;
import rainbowie.RequirementList;

/**
 * Defines the Pair Wise comparisons technique used.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class Pairwise {
    private ComparisonList comparisonList;
    private RequirementList rList;
    
    private Graph graph;
    
    private int currentRow;
    /* The 15 first random indices (RI) values */
    private double[] riValues = {0.0, 0.0, 0.58, 0.90, 1.12, 
            1.24, 1.32, 1.41, 1.45, 1.49, 
            1.51, 1.48, 1.56, 1.57, 1.59 };
    private double[] rowSums;
    private double[][] valueMatrix;
    private double CR; // Consistency Ratio
    /* 
     * allowedPaths decides the number of paths to use when using a 
     * random sample of paths.
     * 
     * maxPathsBeforeRandom decides the threshold where no longer
     * all possible paths shall be used, instead the random sample
     * is used.
     */
    private int allowedPaths = 100, maxPathsBeforeRandom = 1000;
    
    /**
     * Constructor for the Pair Wise comparisons technique.
     * 
     * @param rList the RequirementList containing the Requirements to be 
     * prioritized 
     */
    public Pairwise(RequirementList rList) {
        this.rList = rList;
        valueMatrix = new double[rList.getLength()][rList.getLength()];
        graph = new Graph(rList);
        setMatrix();
        setupComparisonList();
        currentRow = 0;
        CR = 0;
    }
    
    /**
     * Sets the number of paths to use when using a 
     * random sample of paths.
     * 
     * @param paths the number of paths to allow
     */
    public void setAllowedPaths(int paths) {
        allowedPaths = paths;
    }
    /**
     * Gets the number of paths to use when using a 
     * random sample of paths
     * 
     * @return the number of paths to allow
     */
    public int getAllowedPaths() {
        return allowedPaths;
    }
    
    /**
     * Sets the threshold where no longer
     * all possible paths shall be used, instead the random sample
     * is used.
     * 
     * @param paths the number of paths to allow
     */
    public void setMaxPathsBeforeRandom(int paths) {
        maxPathsBeforeRandom = paths;
    }
    /**
     * Gets the threshold where no longer
     * all possible paths shall be used, instead the random sample
     * is used.
     * 
     * @return the number of paths to allow
     */
    public int getMaxPathsBeforeRandom() {
        return maxPathsBeforeRandom;
    }
    
    /**
     * Specialized method for matrix-vector multiplication.
     * 
     * @param matrix the matrix
     * @param vector the vector
     * @return       the resulting vector
     */
    private double[] matrixMultiply(double[][] matrix, double[] vector) {
        double [] result = new double[vector.length];
        for(int row=0;row<vector.length;row++) {
            for(int col=0;col<matrix[0].length;col++) {
                result[row] = result[row] + matrix[row][col]*vector[col];
            }
        }
        return result;
    }
    
    /**
     * Creates an identity matrix.
     */
    private void setMatrix() {
        for(int r=0;r<valueMatrix.length;r++) {
            for(int c=0;c<valueMatrix[0].length;c++) {
                if(r==c) {
                    valueMatrix[r][c] = 1.0; 
                }
                else {
                    valueMatrix[r][c] = 0.0;
                }
            }                
        }
    }
    
    /**
     * Sets up the comparisons and adds them to a list in a random order.  
     */
    private void setupComparisonList() {
        ComparisonList tempHeadList = new ComparisonList();
        ComparisonList tempTailList = new ComparisonList();
        
        /* 
         * Create a copy of the rList to use
         * for random selection of the n-1
         * required comparisons 
         */ 
        RequirementList randomList = rList.getClampedCopy();
        
        int req1Index = (int) (Math.random()*(randomList.getLength()));
        Requirement req1 = randomList.getReq(req1Index);
        randomList.removeReq(req1);
        
        int req2Index = -1;
        Requirement req2 = null;
        
        /* Save the first requirement */
        Requirement reqFirst = req1;
        
        
        /* 
         * Randomly select the n-1 required comparisons
         */
        while(randomList.getLength()>0) {
            req2Index = (int) (Math.random()*(randomList.getLength()));
            req2 = randomList.getReq(req2Index);
            tempHeadList.addRow(rList.getIndexOf(req1), req1, rList.getIndexOf(req2),
                    req2, 0.0);
            randomList.removeReq(req2);
            req1Index = req2Index;
            req1 = req2;
        }
        
        
        tempHeadList.addRow(rList.getIndexOf(req2), req2, 
                rList.getIndexOf(reqFirst), reqFirst, 0.0);
        
        /*
         * Create the rest of the comparisons
         */
        for(int r1=0;r1<rList.getLength();r1++) {
            if (rList.getReq(r1).getReqNbr().length() > 0) {
                for(int r2=r1+1;r2<rList.getLength();r2++) {
                    if (rList.getReq(r2).getReqNbr().length() > 0) {
                        /* 
                         * Check if the current comparison pair already is added 
                         * among the required comparisons 
                         */
                        if(!tempHeadList.rowExists(rList.getReq(r1), rList.getReq(r2))) {
                            if (Math.random()<0.5) { 
                                tempTailList.addRow(r1, rList.getReq(r1), r2, 
                                        rList.getReq(r2), 0.0);
                            }
                            else {
                                tempTailList.addRow(r2, rList.getReq(r2), r1, 
                                        rList.getReq(r1), 0.0);
                            }
                        }
                    }
                }
            }
        }
        
        /*
         * First randomly add the required comparisons to the final list
         * Then randomly add the rest of the comparisons
         */
        comparisonList = new ComparisonList();
        int head = tempHeadList.getLength();
        int tail = tempTailList.getLength();
        int n =  head + tail; 
        for(int i=0;i<n;i++) {            
            if(i<head) {
                comparisonList.addRow(tempHeadList.removeRow(
                        (int) (Math.random()*tempHeadList.getLength())));
            }
            else {
            	comparisonList.addRow(tempTailList.removeRow(
            	        (int) (Math.random()*tempTailList.getLength())));
            }
        }
    }
    
    /**
     * Backs the list one step and returns the value found there.
     * 
     * @return the value at one step back
     */
    public int backOneStep() {
        currentRow--;
        return getCurrentValue();
    }
    
    /**
     * Returns the current row in the matrix as an array.
     * 
     * @return the current row in the matrix
     */
    public Object[] getCurrentComparisonRow() {
        return comparisonList.getRow(currentRow);
    }
    
    /**
     * Gets the index of the current row.
     * 
     * @return the index of the current row
     */
    public int getCurrentRowIndex() {
        return currentRow;
    }
    
    /**
     * Gets the value of the current comparison.
     * 
     * @return the value of the current comparison
     */
    public int getCurrentValue() {
        double val = ((Double) comparisonList.getRow(currentRow)[4]).doubleValue();
        if(val == 0) {
            return 0;
        }
        else if(val<1) {
            /* Convert to an integer */
            val = 1/val;
            val = -1.0 * val;
            return (int) val;
        }
        else {
            return (int) val;
        }
    }
    
    /**
     * Gets the current consistency ratio
     * 
     * @return the current consistency ratio with two decimals
     */
    public double getCR() {
        return ((int) (CR*100))/100.0;
    }

    
    /**
     * Sets the values for the current criteria and calculates the priorities 
     * using to AHP.
     * 
     * @param currentCriteria the current criteria
     */
    public void prioDone(String currentCriteria) {
        
        /* Set the values in the matrix */
        for(int i=0;i<comparisonList.getLength();i++) {
            Object[] row = comparisonList.getRow(i);
            int req1 = ((Integer) row[0]).intValue();
            int req2 = ((Integer) row[2]).intValue();
            double value = ((Double) row[4]).doubleValue();
            if (value != 0.0) {
                graph.addArc((Requirement) row[1], (Requirement) row[3],
                        ((Double) row[4]).doubleValue());
            }
            if(value==0.0) {
                value = graph.getArcValue((Requirement) row[1], (Requirement) row[3]);
            }
            valueMatrix[req1][req2] = value;
            valueMatrix[req2][req1] = 1 / value;
        }
        graph.checkForMethodChange();
        
        double[][] normalizedValueMatrix = 
            new double[valueMatrix.length][valueMatrix[0].length];
        
        /* Averaging over normalized columns */ 
        for(int col=0;col<valueMatrix[0].length;col++) {
            double sum = 0.0;
            for(int row=0;row<valueMatrix.length;row++) {
                sum = sum + valueMatrix[row][col];
            }
            for(int row=0;row<valueMatrix.length;row++) {
                normalizedValueMatrix[row][col] = valueMatrix[row][col] / sum;
            }
        }
        
        /* 
         * Calculate rowsums to estimate the matrix' eigenvalues and divide 
         * by the number for elements to get the percentages of the 
         * prioritization 
         */
        rowSums = new double[normalizedValueMatrix.length];
        for(int row=0;row<normalizedValueMatrix.length;row++) {
            double sum = 0.0;
            for(int col=0;col<normalizedValueMatrix[0].length;col++) {
                sum = sum + normalizedValueMatrix[row][col];
            }
            /* Divide by number of elements */
            rowSums[row] = sum / rowSums.length;

            if(currentCriteria.compareTo(rList.getPrioCriteria1())==0) {
                rList.getReq(row).setPriority1(rowSums[row]);
            }
            else {
                rList.getReq(row).setPriority2(rowSums[row]);
            }
        }
        
        /* Consistency check */
        double[] lambdaMax = matrixMultiply(valueMatrix, rowSums);
        double lambdaMaxSum = 0.0;
        for(int i=0;i<lambdaMax.length;i++) {
            lambdaMax[i] = lambdaMax[i] / rowSums[i];
            lambdaMaxSum = lambdaMaxSum + lambdaMax[i];        
        }
        lambdaMaxSum = lambdaMaxSum / lambdaMax.length;
        /* Consistency Index (CI) */
        double CI = (lambdaMaxSum - lambdaMax.length)/(lambdaMax.length - 1);
        /* Consistency Ratio (CR) */
        CR = 0.0;
        if(lambdaMax.length-1 >= 15) {
            CR = CI / 1.59;
        }
        else {
            CR = CI / riValues[lambdaMax.length-1];
        }    
    }
    
    /**
     * Resets/restarts the Pair Wise method by clearing all (needed) values.
     */
    public void reset() {
        valueMatrix = new double[rList.getLength()][rList.getLength()];
        setMatrix();
        setupComparisonList();
        currentRow = 0;
        graph = new Graph(rList);
    }
    
    /**
     * Sets the value for the current prioritization and checks
     * if there are more comparisons to be done.
     * 
     * @param val the value to set
     * @return    true if there are more prioritizations to be done
     */
    public boolean setValue(double val) {
        comparisonList.getRow(currentRow)[4] = new Double(val);
        currentRow++;
        return currentRow < comparisonList.getLength();
    }
    
    /**
     * Represents the list containing the comparisons.
     * <br><br>
     * The comparison between two requirements are represented in an array
     * that we call a row, each row contains information about the two
     * requirements and the value of the prioritization between them. A row 
     * has the format: row[index1][req1][index2][req2][value] where
     * <li>index1 - is the position of the first Requirement in the 
     * RequirementList
     * <li>req1 - is the first Requirement
     * <li>index2 - is the position of the second Requirement in the 
     * RequirementList
     * <li>req2 - is the second Requirement
     * <li>value - is the value of the prioritization between req1 and req2
     */
    class ComparisonList {
        private LinkedList theList;
        
        /**
         * Creates a new ComparisonList.
         */
        public ComparisonList() {
            theList = new LinkedList();
        }

        /**
         * Sets a row/array that represents a pair of requirements
         * to be compared.
         * 
         * @param index1 the RequirementList index of the first Requirement 
         * @param req1   the first Requirement
         * @param index2 the RequirementList index of the second Requirement 
         * @param req2   the second Requirement
         * @param value  the result of the comparison. Will always be 0.0
         * here since no comparisons have been done yet.
         */
        public void addRow(int index1, Requirement req1, int index2, 
                Requirement req2, double value) {
            Object[] row = new Object[5];
            row[0] = new Integer(index1);
            row[1] = req1;
            row[2] = new Integer(index2);
            row[3] = req2;
            row[4] = new Double(value);
            theList.add(row);
        }
        
        /**
         * Adds an element (row/array of elements) that represents 
         * one comparison, to the list. 
         */
        public void addRow(Object[] row) {
            theList.add(row);
        }
       
        /**
         * Gets the number of elements in the list.
         * 
         * @return the number of elements in the list
         */
        public int getLength() {
            return theList.size();
        }
        
        /**
         * Gets the element at the specified position in the list
         * 
         * @param i the index of the element to return
         * @return  the element representing the given comparison
         */
        public Object[] getRow(int i) {
            return (Object[]) theList.get(i);
        }
        
        /**
         * Removes the element at the specified position in the list
         * 
         * @param i the index of the element to remove
         * @return the removed element
         */
        public Object[] removeRow(int i) {
            return (Object[]) theList.remove(i);
        }
        
        /**
         * Checks if a comparison between the two specified requirements
         * exists in the list
         * 
         * @param r1 the first requirement
         * @param r2 the second requirement
         * @return true if the comparison exists, false if not
         */
        public boolean rowExists(Requirement r1, Requirement r2) {
            for(int i=0;i<theList.size();i++) {
                String reqNbr1 = ((Requirement) (((Object[]) theList.get(i))[1])).getReqNbr();
                String reqNbr2 = ((Requirement) (((Object[]) theList.get(i))[3])).getReqNbr();
                if((reqNbr1.compareTo(r1.getReqNbr())==0 && reqNbr2.compareTo(r2.getReqNbr())==0 ) ||
                        (reqNbr2.compareTo(r1.getReqNbr())==0 && reqNbr1.compareTo(r2.getReqNbr())==0)) {
                    return true;
                }
            }
            return false;
        }
    }

    /**
     * Defines a graph used for estimating missig comparisons when using
     * incomplete pairwise prioritization
     */
    class Graph {
        private Node[] nodes;
        private int nbrOfPaths;
        private int maxPathsFound;
        private PathValue pathProduct;
        private boolean random;
        
        
        /**
         * Constructs the nodes for the graph in which 
         * each node represents a requirement 
         * 
         * @param rList the list of requirements to create nodes for
         */
        public Graph(RequirementList rList) {
            nbrOfPaths = 0;
            maxPathsFound = 0;
            pathProduct = new PathValue();
            random = false;
            nodes = new Node[rList.getLength()];
            for(int i=0;i<nodes.length;i++) {
                nodes[i] = new Node(rList.getReq(i));
            }
        }
        
        /**
         * Checks if the threshold value has been reached where 
         * it becomes inefficient to use all possible paths when 
         * estimating a missing path.
         */
        private void checkForMethodChange() {
            if(maxPathsFound > maxPathsBeforeRandom) {
                random = true;
            }
        }
        
        /**
         * Adds a comparison represented as a directed arc in the graph
         * 
         * @param from the requirement (node) to add the arc from
         * @param to the requirement (node) to add the arc to
         * @param value the value of the comparison (arc) set by the user
         */
        public void addArc(Requirement from, Requirement to, double value) {
            Node f = null;
            Node t = null;
            /* Find the from- and to-node */
            for(int i=0;i<nodes.length;i++) {
                if(nodes[i].getRequirement().getReqNbr().compareTo(from.getReqNbr()) == 0) {
                    f = nodes[i];
                }
                if(nodes[i].getRequirement().getReqNbr().compareTo(to.getReqNbr()) == 0) {
                    t = nodes[i];
                }
            }
            boolean exists = false;
            for(int i=0;i<f.arcs.size();i++) {
                /* Check if the arc already exists */
                if ( ((Arc) f.arcs.get(i)).getTo().equals(t) ) {
                    /* 
                     * Check if the arc already exists with another value, 
                     * if so, update the value.
                     */
                    if(((Arc) f.arcs.get(i)).getValue() != value) {
                        ((Arc) f.arcs.get(i)).setValue(value);
                        t.getArcTo(f).setValue(1.0/value);
                    }                  
                    exists = true;
                }
            }
            /* Add the new arc */
            if (!exists) {
                f.addArc(new Arc(f, t, value));
                t.addArc(new Arc(t, f, 1.0/value));
            }
        }
        
        /**
         * Gets the value of the comparison (arc) between the 
         * specified requirements (nodes)
         * 
         * @param from 	the from-requirement (node)
         * @param to 	the to-requirement (node)
         * @return 		the the value of the comparison (arc)
         */
        public double getArcValue(Requirement from, Requirement to) {
            Node f = null;
            Node t = null;
            for(int i=0;i<nodes.length;i++) {
                if (nodes[i].getRequirement().getReqNbr().compareTo(from.getReqNbr()) == 0) {
                    f = nodes[i];
                }
                if(nodes[i].getRequirement().getReqNbr().compareTo(to.getReqNbr()) == 0) {
                    t = nodes[i];
                }
            }
            if(random) {
                return calculateRandomGeometricMean(f, t);
            }
            return calculateGeometricMean(f, t);
        }
        
        /**
         * Calculates and returns the geometric mean of a random sample of paths
         * between the specified nodes
         * 
         * @param from 	the from-node
         * @param to	the to-node
         * @return		the calculated geometric mean
         */
        public double calculateRandomGeometricMean(Node from, Node to) {
            int i = (int) (Math.random()*from.arcs.size());
            from.visited = true;
            /*
             * For a random outgoing arc from the startnode, make a recursive
             * call to get the pathproducts of all possible paths.
             * Do this until the number of allowed paths has been reached.
             */
            while(nbrOfPaths<allowedPaths) {
                recRandom(((Arc) from.arcs.get(i)).getTo(), to.getRequirement(), 
                        ((Arc) from.arcs.get(i)).getValue());
            }
            from.visited = false;
            double geometricMean = pathProduct.getPathValue();
            pathProduct = new PathValue(); // reset the path value
            nbrOfPaths = 0;
            return geometricMean;
        }
        
        /**
         * Calculates and returns the geometric mean of a all possible paths
         * between the specified nodes
         * 
         * @param from 	the from-node
         * @param to	the to-node
         * @return		the calculated geometric mean
         */
        public double calculateGeometricMean(Node from, Node to) {
            from.visited = true;
            /*
             * For every outgoing arc from the startnode, make a recursive
             * call to get the pathproducts of all possible paths.
             */
            for(int i=0;i<from.arcs.size();i++) {
                rec(((Arc) from.arcs.get(i)).getTo(), to.getRequirement(), ((Arc) from.arcs.get(i)).getValue());
            }
            from.visited = false;
            double geometricMean = pathProduct.getPathValue();
            maxPathsFound = (nbrOfPaths > maxPathsFound ? nbrOfPaths : maxPathsFound);
            pathProduct = new PathValue(); // reset the path value
            nbrOfPaths = 0;
            return geometricMean;
        }
        
        /**
         * Recursive algorithm used to calculate the pathproduct for a randomized 
         * path from a random outgoing arc from the start node.
         * 
         * @param node the start node in the recursive call
         * @param to the requirement (node) that is to be found
         * @param value the value of the path so far
         */
        private void recRandom(Node node, Requirement to, double value)  {
            /* Check if the goal node is found */
            if(node.getRequirement().equals(to)) {
                pathProduct.multiplyPathValue(value);
                nbrOfPaths++;
            }
            /* Check if a dead end node is found */
            else if(node.arcs.size()==1) {
                /*  Do nothing */
                /* This block exists because of readability reasons */ 
            }
            /* Take another step along the path */
            else {
                node.visited = true;
                LinkedList tempList = (LinkedList) node.arcs.clone();
                int i = (int) (Math.random()*tempList.size());
                /* Remove arcs leading to already visited nodes */
                while (tempList.size() > 0 && 
                        ((Arc) tempList.get(i)).getTo().visited == true ) {
                    tempList.remove(i);
                    i = (int) (Math.random()*tempList.size());
                }
                /* Randomly pick the next arc to follow and do a recursive call */
                if (tempList.size() > 0) {
                    recRandom(((Arc) tempList.get(i)).getTo(), 
                            to, value*((Arc) tempList.get(i)).getValue());
                }
               
                node.visited = false;
            }
        }
        
        /**
         * Recursive algorithm used to calculate the pathproduct for all possible 
         * paths from an outgoing arc from the start node.
         * 
         * @param node the start node in the recursive call
         * @param to the requirement (node) that is to be found
         * @param value the value of the path so far
         */
        private void rec(Node node, Requirement to, double value)  {
            /* Check if the goal node is found */
            if(node.getRequirement().equals(to)) {
                pathProduct.multiplyPathValue(value);
                nbrOfPaths++;
            }
            /* Check if a dead end node is found */
            else if(node.arcs.size()==1) {
                /* Do nothing */
                /* This block exists because of readability reasons */  
            }
            /* Take another step along the path */
            else {
                node.visited = true;
                /* For all outgoing arcs... */
                for(int i=0;i<node.arcs.size();i++) {
                    /* ...except those that lead to already visited nodes... */
                    if(((Arc) node.arcs.get(i)).getTo().visited == false ) {
                        /* ...make a new recursive call */
                        rec(((Arc) node.arcs.get(i)).getTo(), to, value*((Arc) node.arcs.get(i)).getValue());
                    }
                }
                node.visited = false;
                
            }
        }
        
        /**
         * Defines a node in the graph
         */
        class Node {
            
            private Requirement req;
            private LinkedList arcs;
            private boolean visited;
            
            /**
             * Constructs a node corresponding to the specified requirement
             * 
             * @param req the requirement to create a node for
             */
            public Node(Requirement req) {
                arcs = new LinkedList();
                visited = false;
                this.req = req;
            }
            
            /**
             * Adds an outgoing arc from the node
             * 
             * @param arc the arc to add
             */
            public void addArc(Arc arc) {
                arcs.add(arc);
            }
            
            /**
             * Get this node's associated  requirement
             * 
             * @return the requirement associated with this node
             */
            public Requirement getRequirement() {
                return req;
            }
            
            /**
             * Gets the arc from this node to the specified node
             * 
             * @param node the node to find the arc to
             * @return the arc if it is found, null if not
             */
            public Arc getArcTo(Node node) {
                Arc arc = null;
                for(int i=0;i<arcs.size();i++) {
                    if(((Arc) arcs.get(i)).getTo().equals(node)) {
                        arc = (Arc) arcs.get(i);
                    }
                }
                return arc;
            }
        }
        
        /**
         * Defines an arc in the graph
         */
        class Arc {
            
            private Node from, to;
            private double value;
            
            /**
             * Constructs an arc between the two specified nodes with the 
             * specified value
             * 
             * @param from 	the from-node
             * @param to	the to-node
             * @param value the specified value
             */
            public Arc(Node from, Node to, double value) {
                this.from = from;
                this.to = to;
                this.value = value;
            }
            
            /**
             *  Gets the node from where this arc leads
             * 
             * @return the from-node
             */
            public Node getFrom() {
                return from;
            }
            
            /**
             *  Gets the node to where this arc leads
             * 
             * @return the to-node
             */
            public Node getTo() {
                return to;
            }
            
            /**
             * Gets the value of this arc (comparison)
             * 
             * @return the value of the arc
             */
            public double getValue() {
                return value;
            }
            
            /**
             * Sets the value of this arc (comparison)
             * 
             * @param value the value to set
             */
            public void setValue(double value) {
                this.value = value;
            }
        }
        
        /**
         * Represents the value of a path in the graph
         */
        class PathValue {
            private LinkedList valueList;
            
            /**
             * Constructs a path value 
             */
            public PathValue() {
                valueList = new LinkedList();
            }
            
            /**
             * Multiplies the path value so far with the specified value
             * 
             * @param value the value to multiply with
             */
            public void multiplyPathValue(double value) {
                valueList.add(new Double(value));
            }
            
            /**
             * Gets the product of the multiplied path value
             * 
             * @return the value of the path
             */
            public double getPathValue() {
                double result = 1.0;
                int size = valueList.size();
                for(int i=0;i<size;i++) {
                    result = Math.pow( ((Double)valueList.get(i)).doubleValue(), 1.0/(size * 1.0) );
                }
                return result;
            }
        }
    }
}
