/*
 * PlanningGame.java
 *
 * Created on 2005-jan-31
 */

package prioritizer;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Toolkit;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Random;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import rainbowie.Requirement;
import rainbowie.RequirementList;
import util.Util;

/**
 * Defines the Planning Game prioritization method.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class PlanningGame {
	
	public static final int CARD_WIDTH = 200, CARD_HEIGHT = 60;
	public static final int HIGH = 0, MEDIUM = 1, LOW = 2, DESK = -1;

	private LinkedList cardsOnDeskList, cardsInCurrentPile;
	private JPanel desk;
	private int deskWidth, deskHeight;
	private int dx, dy;
	private Pile high, medium, low;
	private int oldX, oldY;
	private Random rand;
	private RequirementList rList;
	private Card selectedCard;
	/* Used when selecting more than one Card */
	private LinkedList selectedCardsList; 
	
	/**
	 * Constructor for the Planning Game technique.
	 * 
	 * @param rList the RequirementList containing the Requirements to be 
     * prioritized 
	 */
	public PlanningGame(RequirementList rList) {
		high = new Pile("High");
		medium = new Pile("Medium");
		low = new Pile("Low");
		cardsOnDeskList = new LinkedList();
		selectedCardsList = new LinkedList();
		this.rList = rList;
		selectedCard = null;
		rand = new Random();
	}
	
	/**
	 * Highlights a Card
	 * 
	 * @param c the Card to highlight
	 */
	private void highlight(Card c) {
		if (c != null) {
			c.highlighted = true;
		}
	}
	
	/**
	 * Highlights a Pile.
	 * 
	 * @param p the Pile to highlight
	 */
	private void highlight(Pile p) {
		if (p != null) {
			p.highlighted = true;
		}
	}
	
	/**
	 * Sets a random position on the screen, for the Card to be drawn at
	 * 
	 * @param c the Card to set the randomized position for
	 */
	private void setRandomCardPosition(Card c) {
		Point p = new Point(rand.nextInt(deskWidth-400),
				rand.nextInt(deskHeight-50));
		c.setPos(p);
	}
	
	/**
	 * Unhighlights a Card.
	 * 
	 * @param c the Card to unhighlight
	 */
	private void unhighlight(Card c) {
		if (c != null) {
			c.highlighted = false;
		}
	}
	
	/**
	 * Unhighlights all cards
	 */
	private void unhighlightAndUnselectAllCards() {
	    for (int i=0;i<cardsOnDeskList.size();i++) {
	        unhighlight((Card) cardsOnDeskList.get(i));
	    }
	    for (int i=0;i<high.cardsList.size();i++) {
	        unhighlight((Card) high.cardsList.get(i));
	    }
	    for (int i=0;i<medium.cardsList.size();i++) {
	        unhighlight((Card) medium.cardsList.get(i));
	    }
	    for (int i=0;i<low.cardsList.size();i++) {
	        unhighlight((Card) low.cardsList.get(i));
	    }
	    selectedCardsList.clear();
	}
	
	/**
	 * Unhighlists a Pile.
	 * 
	 * @param p the Pile to unhighlight
	 */
	private void unhighlight(Pile p) {
		if (p != null) {
			p.highlighted = false;
		}
	}
	
	/**
	 * Deselects the selected Card.
	 */
	public void deselectCard() {
		unhighlight(selectedCard);
		selectedCard = null;
	}
	
	/**
	 * Draws all the cards and piles on the desk
	 * 
	 * @param g the graphics component to use for drawing
	 * @param c the panel (desk) to draw upon
	 */
	public void drawDesk(Graphics g, JPanel c) {
		this.desk = c;
		high.draw(g);
		medium.draw(g);
		low.draw(g);
		
		ListIterator itr = cardsOnDeskList.listIterator();
		for(int i=0;i<cardsOnDeskList.size();i++) {
			((Card) itr.next()).draw(g);
			
		}
	}
	
	/**
	 * Draws the cards in the specified Pile.
	 * 
	 * @param g        the graphics component to use for drawing
	 * @param pileType the pile to be drawn, HIGH, MEDIUM or LOW 
	 */
	public void drawPile(Graphics g, int pileType) {
		switch (pileType) {
			case HIGH:
				cardsInCurrentPile = high.cardsList;
			break;
			case MEDIUM:
				cardsInCurrentPile = medium.cardsList;
			break;
			case LOW:
				cardsInCurrentPile = low.cardsList;
			break;
		}
		if (cardsInCurrentPile != null) {
			ListIterator itr = cardsInCurrentPile.listIterator();
			for(int i=0;i<cardsInCurrentPile.size();i++) {
				((Card) itr.next()).draw(g);
			}	
		}
	}
	
	/**
	 * Gets the horizontal distance between the left edge of the card and the 
	 * clicked position.
	 * 
	 * @return the horizontal distance
	 */
	public int getDx() {
		return dx;
	}
	
	/**
	 * Gets the vertical distance between the upper edge of the card and the 
	 * clicked position.
	 * 
	 * @return the vertical distance
	 */
	public int getDy() {
		return dy;
	}
		
	/**
	 * Gets the number of cards in he currently selected Pile.
	 * 
	 * @return the number of cards in the selected Pile; 
	 * 0 if no Pile is selected
	 */
	public int getNbrOfCardsInCurrentPile() {
		if (cardsInCurrentPile != null) {
			return cardsInCurrentPile.size();
		}
		return 0;
	}
	
	/**
	 * Gets the number of cards on the desk.
	 * 
	 * @return the number of cards on the desk
	 */
	public int getNbrOfCardsOnDesk() {
		return cardsOnDeskList.size();
	}
	
	/**
	 * Gets the LinkedList representing the specified Pile.
	 * 
	 * @param pileName the specified Pile
	 * @return         the list containing the cards in the 
	 * specified Pile
	 */
	public LinkedList getPile(String pileName) {
		if(pileName.compareTo("High")==0) {
			return high.cardsList;
		}
		else if(pileName.compareTo("Medium")==0) {
			return medium.cardsList;
		}
		else if(pileName.compareTo("Low")==0) {
			return low.cardsList;
		}
		else {
			return null;
		}
	}
	
	/**
	 * Gets the Requirement at the specified index in the specified Pile.
	 * 
	 * @param pileName the Pile to ge the Requirement from
	 * @param i        the index of the requirement in the Pile
	 * @return		   the sought requirement if found; null if not found
	 */
	public Requirement getRequirement(String pileName, int i) {
	    LinkedList tempList;
	    if(pileName.compareTo("High")==0) {
			tempList = high.cardsList;
		}
		else if(pileName.compareTo("Medium")==0) {
			tempList = medium.cardsList;
		}
		else if(pileName.compareTo("Low")==0) {
			tempList = low.cardsList;
		}
		else {
		    return null;
		}
	    
	    try {
			return ((Card) tempList.get(i)).getRequirement();
		}
		catch (IndexOutOfBoundsException ioobe) {
			return null;
		}
	}
	
	/**
	 * Gets the list containing all the requirements to be prioritized.
	 * 
	 * @return the list containing all the requirements to be prioritized
	 */
	public RequirementList getRequirementList() {
		return rList;
	}
	
	/**
	 * Create a Card for each Requirement and assign a random position, within
	 * the specified drawing area limits.
	 * 
	 * @param deskWidth  the width of the drawing area
	 * @param deskHeight the height of the drawing area
	 */
	public void makeCards(int deskWidth, int deskHeight) {
		this.deskWidth = deskWidth;
		this.deskHeight = deskHeight;
		Requirement r;
		for(int i=0;i<rList.getLength();i++) {
			if ((r = rList.getReq(i)) != null && (r.getReq().length() > 0 ||
			        r.getReqNbr().length() > 0)) {
			    Card c = new Card(r);		    
				setRandomCardPosition(c);
				cardsOnDeskList.add(c);
			}
		}
	}
	
	/**
	 * Moves away a card to an empty place in the pile.
	 * <br><br>
	 * Used in the Pile when moving cards.
	 */
	public void moveCardToEmptyPlace() {
		if(selectedCard!=null) {
			selectedCard.setPos(new Point(oldX, oldY));
		}
	}
	
	/**
	 * Moves the selected card to a new position in the Pile when 
	 * dragged & dropped. Also makes sure that cards are "snapped" 
	 * into the right positions when dropped.
	 * 
	 * @param y        the vertical position where the card is dropped
	 * @param pileType the pile in which the card is being moved
	 */
	public void moveSelectedInPileTo(int y, int pileType) {
		switch(pileType) {
		case HIGH :
			cardsInCurrentPile = high.cardsList;
			break;
		case MEDIUM :
			cardsInCurrentPile = medium.cardsList;
			break;
		case LOW :
			cardsInCurrentPile = low.cardsList;
			break;
		}
		
		if(y-dy<25) {
			y = 25 + dy;
		}
		if(y-dy > (cardsInCurrentPile.size()-1)*(CARD_HEIGHT+20)+ 25) {
			y = (cardsInCurrentPile.size()-1)*(CARD_HEIGHT+20) + dy + 25;
		}
		if (selectedCard != null) {
			selectedCard.setPos(new Point(selectedCard.pos.x, y-dy));
			ListIterator itr = cardsInCurrentPile.listIterator();
			while (itr.hasNext()) {
				Card c = (Card) itr.next();
				if ( c.selectedBy(new Point(selectedCard.pos.x, y-dy+CARD_HEIGHT/2)) 
						&& !c.equals(selectedCard)) {
					int justMovedFromX = c.pos.x;
					int justMovedFromY = c.pos.y;
					c.setPos(new Point(oldX, oldY));
					oldX = justMovedFromX;
					oldY = justMovedFromY;
					
					itr = null;
					
					/* Move the cards in the list */
					int i = cardsInCurrentPile.indexOf(selectedCard);
					int k = cardsInCurrentPile.indexOf(c);
					Card tmp = (Card) cardsInCurrentPile.remove(k);
					cardsInCurrentPile.add(i, tmp);
					break;
				}
			}
		}
	}

	/**
	 * Moves the selected card to a new position on the desk. Also 
	 * Highlights the correct Pile if the card is dragged over it.
	 * 
	 * @param x the horizontal position of the mouse pointer
	 * @param y the vertical position of the mouse pointer
	 */
	public void moveSelectedTo(int x, int y, Dimension dim) {
		if (selectedCard != null) {
		    /* 
		     * Stop the cards from being dragged off the edges of the screen. 
		     */
			if(x-dx<0) {
				x = 0 + dx;
			}
			if(x-dx+PlanningGame.CARD_WIDTH>dim.width) {
				x = desk.getWidth() - PlanningGame.CARD_WIDTH + dx;
			}
			if(y-dy<0) {
				y = 0 + dy;
			}
			if(y-dy+PlanningGame.CARD_HEIGHT>dim.height) {
				y = desk.getHeight() - PlanningGame.CARD_HEIGHT + dy;
			}
			/* Move card */
			selectedCard.setPos(new Point(x-dx, y-dy));
			
			/* Check if the card is over a pile */
			if(high.selectedBy(new Point(x-dx+CARD_WIDTH/2, y-dy+CARD_HEIGHT/2))) {
				unhighlightAllPiles();
				highlight(high);
			}
			else if(medium.selectedBy(new Point(x-dx+CARD_WIDTH/2, y-dy+CARD_HEIGHT/2))) {
				unhighlightAllPiles();
				highlight(medium);
			}
			else if(low.selectedBy(new Point(x-dx+CARD_WIDTH/2, y-dy+CARD_HEIGHT/2))) {
				unhighlightAllPiles();
				highlight(low);
			} else {
				unhighlightAllPiles();
			}
		}
	}
	
	/**
	 * Moves all cards in the selection square when they are dragged.
	 * 
	 * @param x        the x-position of the mouse pointer
	 * @param y        the y-position of the mouse pointer
	 * @param deskSize the size of the desk's drawing area
	 */
	public void moveSelectedCardsTo(int x, int y, Dimension deskSize) {
	    if(selectedCardsList.contains(selectedCard)) {
	        int diffX = x - selectedCard.getPos().x;
	        int diffY = y - selectedCard.getPos().y;
	        for(int i=0;i<selectedCardsList.size();i++) {
	            Card c = (Card) selectedCardsList.get(i);
	            if(c.equals(selectedCard)){
	                moveSelectedTo(x, y, deskSize);
	            }
	            else {
	                int newPosX = c.getPos().x+diffX;
	                int newPosY = c.getPos().y+diffY;
	                /* 
	                 * Stop the cards from being dragged off the edges of the screen. 
	                 */
	                if(newPosX-this.dx<0) {
	                    newPosX = 0 + this.dx;
	                }
	                if(newPosX-this.dx+PlanningGame.CARD_WIDTH>deskSize.width) {
	                    newPosX = deskSize.width - PlanningGame.CARD_WIDTH + this.dx;
	                }
	                if(newPosY-this.dy<0) {
	                    newPosY = 0 + this.dy;
	                }
	                if(newPosY-this.dy+PlanningGame.CARD_HEIGHT>deskSize.height) {
	                    newPosY = deskSize.height - PlanningGame.CARD_HEIGHT + this.dy;
	                }
	                c.setPos(new Point(newPosX-this.dx, newPosY-this.dy));
	            }
	        }
	    }
	    else {
	        moveSelectedTo(x, y, deskSize);
	    }
	}
	
	/**
	 * Removes the selected card from the desk and adds it to the highlighted 
	 * pile. Also sets a new position for the card in the Pile.
	 * 
	 * @return true if the move suceeded; else false
	 */
	public boolean putCardInPile() {
		if (selectedCard != null) {
			if(high.highlighted) {
			    for(int i=0;i<selectedCardsList.size();i++) {
			        Card c = (Card) selectedCardsList.get(i);
			        c.setPos(new Point(25, 25+high.cardsList.size()*(CARD_HEIGHT+20)));
			        high.addCard(c);
			        cardsOnDeskList.remove(c);
			        unhighlight(c);
			    }
				selectedCard = null;
				selectedCardsList.clear();
				return true;
			}
			else if(medium.highlighted) {
			    for(int i=0;i<selectedCardsList.size();i++) {
			        Card c = (Card) selectedCardsList.get(i);
			        c.setPos(new Point(25, 25+medium.cardsList.size()*(CARD_HEIGHT+20)));
			        medium.addCard(c);
			        cardsOnDeskList.remove(c);
			        unhighlight(c);
			    }
				selectedCard = null;
				selectedCardsList.clear();
				return true;
			}
			else if(low.highlighted) {
			    for(int i=0;i<selectedCardsList.size();i++) {
			        Card c = (Card) selectedCardsList.get(i);
			        c.setPos(new Point(25, 25+low.cardsList.size()*(CARD_HEIGHT+20)));
			        low.addCard(c);
			        cardsOnDeskList.remove(c);
			        unhighlight(c);
			    }
				selectedCard = null;
				selectedCardsList.clear();
				return true;
			} 
		}
		return false;
	}
	
	/**
	 * Tries to select a Card on the desk or in a Pile at the 
	 * specified position.
	 * 
	 * @param x        the horizontal position of the mouse pointer
	 * @param y	 	   the vertical position of the mouse pointer
	 * @param pileType HIGH, MEDIUM, LOW or DESK depending on where the card is
	 * @return		   the Requirement associated with the Card if selection
	 * succeeded; else null
	 */
	public Requirement select(int x, int y, int pileType) {
		ListIterator lItr = null;
		switch (pileType) {
		case HIGH:
			lItr = high.cardsList.listIterator(high.cardsList.size());
		break;
		case MEDIUM:
			lItr = medium.cardsList.listIterator(medium.cardsList.size());
		break;
		case LOW:
			lItr = low.cardsList.listIterator(low.cardsList.size());
		break;
		case DESK: // No pile selected, we are on the desk
			lItr = cardsOnDeskList.listIterator(cardsOnDeskList.size());
		break;		
		}

		unhighlightAllPiles();
		selectedCard = null;
		while (lItr.hasPrevious()) {
			Card c = (Card) lItr.previous();
			if ( c.selectedBy(new Point(x, y)) ) {
				selectedCard = c;
				if(!selectedCardsList.contains(selectedCard)) {
				    unhighlightAndUnselectAllCards();
				    selectedCardsList.add(selectedCard);
				}
				highlight(selectedCard);
				dx = x - c.getPos().x;
				dy = y - c.getPos().y;
				oldX = selectedCard.pos.x;
				oldY = selectedCard.pos.y;
				if(pileType==DESK) {
					cardsOnDeskList.remove(selectedCard);
					cardsOnDeskList.addLast(selectedCard);
				}
				return c.requirement;
			}			
		}
		unhighlightAndUnselectAllCards();
		return null;
	}
	
	public void addCardToSelectionList(Point p) {
	    ListIterator lItr = cardsOnDeskList.listIterator(cardsOnDeskList.size());
	    while (lItr.hasPrevious()) {
			Card c = (Card) lItr.previous();
			if ( c.selectedBy(p) ) {
				if(!selectedCardsList.contains(c)) {
				    selectedCardsList.add(c);
				    break;
				}
			}
	    }
	}
	
	/**
	 * Tries to get the Requirement on the desk or in a Pile at the 
	 * specified position.
	 * 
	 * @param p			the specified position
	 * @param pileType 	HIGH, MEDIUM, LOW or DESK depending on where the card is
	 * @return		   	the Requirement associated with the Card if selection
	 * 					succeeded; else null
	 */
	public Requirement getRequirementAt(Point p, int pileType) {
	    ListIterator lItr = null;
		switch (pileType) {
		case HIGH:
			lItr = high.cardsList.listIterator(high.cardsList.size());
		break;
		case MEDIUM:
			lItr = medium.cardsList.listIterator(medium.cardsList.size());
		break;
		case LOW:
			lItr = low.cardsList.listIterator(low.cardsList.size());
		break;
		case DESK: /* No pile selected, we are on the desk */
			lItr = cardsOnDeskList.listIterator(cardsOnDeskList.size());
		break;		
		}
		while (lItr.hasPrevious()) {
			Card c = (Card) lItr.previous();
			if ( c.selectedBy(p) ) {
				return c.requirement;
			}			
		}
		return null;   
	}

	/**
	 * Selects all the cards withing the user-drawn selection square.
	 * 
	 * @param startPoint upper left corner in the selection square
	 * @param endPoint   lower righti corner in the selection square
	 */
	public void selectCardsInSelectionSquare(Point startPoint, Point endPoint) {
	    int startX = 0;
        int startY = 0;
        int endX = 0;
        int endY = 0;
	    if (startPoint.x<endPoint.x) {
	        startX = startPoint.x;
	        endX = endPoint.x;
	    }
	    else {
	        startX = endPoint.x;
	        endX = startPoint.x;
	    }
	    if (startPoint.y<endPoint.y) {
	        startY = startPoint.y;
	        endY = endPoint.y;
	    }
	    else {
	        startY = endPoint.y;
	        endY = startPoint.y;
	    }
	    
	    ListIterator lItr = cardsOnDeskList.listIterator(cardsOnDeskList.size());
        selectedCardsList.clear();
        while (lItr.hasPrevious()) {
            Card c = (Card) lItr.previous();
            unhighlight(c);
            
            /* +=10 because we do not want to go through every single point */
            for (int x=startX;x<=endX;x+=10) {
                for (int y=startY;y<=endY;y+=10) {
                    if ( c.selectedBy(new Point(x,y)) && !selectedCardsList.contains(c)) {
                        highlight(c);
                        selectedCardsList.add(c);
                        x=endX;
                        y=endY;
                    }
                }
            }
        }
	}
	
	
	/**
	 * Tries to select a pile at the given position.
	 * 
	 * @param x the horizontal position of the mouse pointer
	 * @param y	the vertical position of the mouse pointer
	 * @return	an array in the form of: <br>
	 * [name of the pile][list containing the cards in the pile]		
	 */
	public Object[] selectPile(int x, int y) {
		Object[] pileInfo = new Object[2];
		pileInfo[0] = "None";
		pileInfo[1] = null;
		if(high.selectedBy(new Point(x, y))) {
			pileInfo[0] = "High";
			pileInfo[1] = high.cardsList;
		}
		else if(medium.selectedBy(new Point(x, y))) {
			pileInfo[0] = "Medium";
			pileInfo[1] = medium.cardsList;
		}
		else if(low.selectedBy(new Point(x, y))) {
			pileInfo[0] = "Low";
			pileInfo[1] = low.cardsList;
		} 
		return pileInfo;
	}
	
	/**
	 * Sends the selected card in the specified Pile, to the desk.
	 * 
	 * @param pileType the specified Pile
	 */
	public void sendToDesk(int pileType) {
		switch(pileType) {
		case HIGH :
			cardsInCurrentPile = high.cardsList;
			break;
		case MEDIUM :
			cardsInCurrentPile = medium.cardsList;
			break;
		case LOW :
			cardsInCurrentPile = low.cardsList;
			break;
		}
		if (selectedCard != null && cardsInCurrentPile.contains(selectedCard)) {
			cardsOnDeskList.add(selectedCard);
			cardsInCurrentPile.remove(selectedCard);
			setRandomCardPosition(selectedCard);
			unhighlight(selectedCard);
			selectedCard = null;
		}
		for(int i=0;i<cardsInCurrentPile.size();i++) {
			((Card) cardsInCurrentPile.get(i)).pos = new Point(25, 25+i*(CARD_HEIGHT+20));
		}
	}
	
	/**
	 * Rank the requirements by the order in the planning game lists.
	 * The integer part is each requirements priority in the pile and
	 * the decimal part decides which pile it belongs to.
	 * Piles: x.0 = high, x.1 = medium and x.2 = low
	 * e.g. 3.2 means that it is the fourth requirement in the lower pile.
	 * <br><br>Also sets new random positions for the cards 
	 * 
	 * @param criteria       the current prioritization criteria
	 * @param skipPriorities true if priorities are set by $100 method
	 */
	public void setCurrentPrioritiesAndShuffle(String criteria, boolean skipPriorities) {
		Requirement r = null;
		double i = 0.0;
		while(high.cardsList.size()>0) {
			Card c = (Card) high.cardsList.removeFirst();
			r = c.getRequirement();
			if(!skipPriorities) {
			    if(criteria.compareTo(rList.getPrioCriteria1())==0) {
			        r.setPriority1(i + 0.0);
			    }
			    else {
			        r.setPriority2(i + 0.0);
			    }
			}
			setRandomCardPosition(c);
			cardsOnDeskList.add(c);
			i++;
		}
		i = 0.0;
		while(medium.cardsList.size()>0) {
			Card c = (Card) medium.cardsList.removeFirst();
			r = c.getRequirement();
			if(!skipPriorities) {
			    if(criteria.compareTo(rList.getPrioCriteria1())==0) {
			        r.setPriority1(i + 0.1);
			    }
			    else {
			        r.setPriority2(i + 0.1);
			    }
			}
			setRandomCardPosition(c);
			cardsOnDeskList.add(c);	
			i++;
		}
		i = 0.0;
		while(low.cardsList.size()>0) {
			Card c = (Card) low.cardsList.removeFirst();
			r = c.getRequirement();
			if(!skipPriorities) {
			    if(criteria.compareTo(rList.getPrioCriteria1())==0) {
			        r.setPriority1(i + 0.2);
			    }
			    else {
			        r.setPriority2(i + 0.2);
			    }
			}
			setRandomCardPosition(c);
			cardsOnDeskList.add(c);
			i++;
		}
		i = 0.0;
		unhighlight(selectedCard);
		selectedCard = null;
	}
	
	/**
	 * Unhighlights all piles.
	 */
	public void unhighlightAllPiles() {
		unhighlight(high);
		unhighlight(medium);
		unhighlight(low);
		desk.repaint();
	}
	
	/**
	 * Defines a Card representing a Requirement in the Planning Game method
	 */
	class Card {
		private boolean highlighted;
		private Point pos;
		private Requirement requirement;
		
		private String shortName;
	
		/**
		 * Constructs a Card
		 * 
		 * @param requirement the requirement to create the Card for
		 */
		private Card(Requirement requirement) {
			this.requirement = requirement;
			highlighted = false;
			if (requirement.getReq().length()!=0) {
			    shortName = Util.shortString(requirement.getReq(), 27, 3);
			}
			else {
			    shortName = Util.shortString(requirement.getReqNbr(), 27, 3);
			}
		}
		
		/**
		 * Gets the cards position.
		 * 
		 * @return the upper left corner position of the card
		 */
		private Point getPos() {
			return pos;
		}
		
		/**
		 * Gets the requirement associated with this Card.
		 *  
		 * @return the requirement associated with this Card
		 */
		private Requirement getRequirement() {
			return requirement;
		}
		
		/**
		 * Moves the Card to the specified position.
		 * 
		 * @param pos the specified position to move the Card to
		 */
		private void setPos(Point pos) {
			this.pos = pos;
		}

		/**
		 * Check if the Card is selectable by a click in this position.
		 * 
		 * @param pos mouse pointer position
		 * @return    true if the card is selectable from the specified position
		 */
		private boolean selectedBy(Point pos) {
			if (pos.x >= this.pos.x && pos.x <= this.pos.x+CARD_WIDTH && 
					pos.y >= this.pos.y && pos.y <= this.pos.y+CARD_HEIGHT ) {
				return true;
			}
			return false;
		}
		
		/**
		 * Draws the Card with the specified graphics component
		 * 
		 * @param g the specified graphics component
		 */
		public void draw(Graphics g) {
			g.setColor(new Color(255, 255, 240));
			g.fillRect(getPos().x, getPos().y, CARD_WIDTH, CARD_HEIGHT);
			if (highlighted) {
				g.setColor(Color.RED);
			} else {
				g.setColor(Color.BLACK);
			}
			/* Draw the Card */
			g.drawRect(getPos().x, getPos().y, CARD_WIDTH, CARD_HEIGHT);
			g.setFont(new Font("Monospaced", Font.PLAIN, 12));
			StringTokenizer tokenizer = new StringTokenizer(shortName, "\n");
			int i = 1;
			/* Write the description in the Card */
			while(tokenizer.hasMoreTokens()) {
				g.drawString(tokenizer.nextToken(), getPos().x+10, getPos().y+15*i+3);
				i++;
			}
		}
	}
	
	/**
	 * Defines a Pile where the cards can be put
	 */
	class Pile {
		private static final int WIDTH = 160, HEIGHT = 110;
		
		private LinkedList cardsList;
		private boolean highlighted;
		private String pileType;
		private Point pos;
		private Image image, imageHighlight;
		
		/**
		 * Constructs a Pile
		 * 
		 * @param pileType defines the type of the pile: High, Medium or Low
		 */
		private Pile(String pileType) {
			this.pileType = pileType;
			cardsList = new LinkedList();
			highlighted = false;
			
			if(pileType.compareTo("High")==0) {
			    image = Toolkit.getDefaultToolkit().getImage("images\\high_transp.gif");
			    imageHighlight = Toolkit.getDefaultToolkit().getImage("images\\high_highlight_transp.gif");
			}
			else if(pileType.compareTo("Medium")==0) {
				image = Toolkit.getDefaultToolkit().getImage("images\\medium_transp.gif");
				imageHighlight = Toolkit.getDefaultToolkit().getImage("images\\medium_highlight_transp.gif");
			}
			else {
				image = Toolkit.getDefaultToolkit().getImage("images\\low_transp.gif");
				imageHighlight = Toolkit.getDefaultToolkit().getImage("images\\low_highlight_transp.gif");
			}
			
			/* Make sure the pile images are loaded */
			MediaTracker mediaTracker = new MediaTracker(new Canvas());
			mediaTracker.addImage(image, 0);
			mediaTracker.addImage(imageHighlight, 1);
			try	{
				mediaTracker.waitForID(0);
				mediaTracker.waitForID(1);
				if(mediaTracker.isErrorID(0) || mediaTracker.isErrorID(1)) {
				    throw new InterruptedException("Unable to set box image(s)");
				}
			} 
			catch (InterruptedException ie) {
			    JOptionPane.showMessageDialog(null, "Unable to set box image(s)",
						"Failed", JOptionPane.ERROR_MESSAGE);
				ie.printStackTrace();
			}
		}
		
		/**
		 * Adds a Card to the last position in the Pile
		 * 
		 * @param c the Card to be added
		 */
		private void addCard(Card c) {
			cardsList.add(c);
		}
		
		/**
		 * Draws the the Pile (box on desk) with the specified 
		 * graphics component
		 * 
		 * @param g the specified graphics component
		 */
		public void draw(Graphics g) {
		    Image currentImage;
			if (!highlighted) {
			    currentImage = image;
			} 
			else {
			    currentImage = imageHighlight;
			}
			
			g.setColor(Color.BLACK);
			g.setFont(new Font("Verdana", Font.PLAIN, 12));
			if(pileType.compareTo("High")==0) {
				pos = new Point(desk.getWidth()-200, 50);				
			}
			else if(pileType.compareTo("Medium")==0) {
				pos = new Point(desk.getWidth()-200, 200);
			}
			else {
				pos = new Point(desk.getWidth()-200, 350);
			}
			g.drawImage(currentImage, pos.x, pos.y, null);
			g.drawString(cardsList.size()+"", pos.x+105, pos.y+100);
		}
		
		/**
		 * Check if the Pile is selectable by a click in this position.
		 * 
		 * @param pos mouse pointer position
		 * @return    true if the Pile is selectable from the specified position
		 */
		public boolean selectedBy(Point pos) {
			if (pos.x >= this.pos.x && pos.x <= this.pos.x+WIDTH && 
					pos.y >= this.pos.y && pos.y <= this.pos.y+HEIGHT ) {
				return true;
			} 
			return false;
		}
	}
}
