/*
 * RequirementList.java
 *
 * Created on Jan 21, 2005
 */

package rainbowie;

import java.util.*;

/**
 * Defines a list of requirements.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class RequirementList {

    public static final int  PLANNING_GAME = 0, PAIR_WISE = 1, DOLLAR = 2;
    
	private LinkedList theList;
	private int prioMethod;
	private boolean sortInvert = false;
	private int lastSortColumn = -1;
	
	private String prioCriteria1 = "";
	private String prioCriteria2 = "";
	
	/**
	 * Constructs an empty requirement list
	 */
	public RequirementList() {
		theList = new LinkedList();
	}
	
	/**
	 * Adds a requirement to the last posistion in the list
	 * 
	 * @param req the requirement to add
	 */
	public void addRequirement(Requirement req) {
		theList.add(req);
	}
	
	/**
	 * Returns the requirement, if any, with the given requirement number
	 * 
	 * @param reqNbr the requirement number to find in the list
	 * @return the requirement if it is found in the list, null if not
	 */
	public Requirement getReq(String reqNbr) {
		ListIterator itr = theList.listIterator();
		Requirement foundReq = null;
		Requirement searchReq = null;
		while(itr.hasNext()) {
			searchReq = (Requirement) itr.next();
			if(searchReq.getReqNbr().compareTo(reqNbr)==0) {
				foundReq = searchReq;
				break;
			}
		}
		return foundReq;
	}
	
	/**
	 * Returns the requirement at the specified position in the list
	 * 
	 * @param i the position in the list with i=0 as the first position
	 * @return the requirement if the given position is within the list, 
	 * null if not
	 */
	public Requirement getReq(int i) {
		try {
			return (Requirement) theList.get(i);
		}
		catch(IndexOutOfBoundsException ioobe) {
			return null;
		}
	}
	
	/**
	 * Returns the position whithin the list where the specified requirement 
	 * can be found
	 * 
	 * @param req a reference to the requirement to find
	 * @return the position of the specified requirement if it is founs, 
	 * -1 if not
	 */
	public int getIndexOf(Requirement req) {
	    for(int i=0;i<theList.size();i++) {
	        if(((Requirement) theList.get(i)).getReqNbr().compareTo(req.getReqNbr())==0) {
	            return i;
	        }
	    }
	    return -1;
	}
	
	/**
	 * Returns the list in a matrix format where each row defines a requirement as:<br>
	 * {requirement number, requirement description, release number, 
	 * (first priority, second priority) }
	 * 
	 * @param complete true if priorities shall be included
	 * @return the requirement matrix
	 */
	public String[][] getReqList(boolean complete) {
		String[][] reqList;
		if(complete) {
			reqList = new String[theList.size()][5];
		}
		else {
			reqList = new String[theList.size()][3];
		}
		for(int i=0;i<theList.size();i++) {
			reqList[i][0] = ((Requirement) theList.get(i)).getReqNbr();
			reqList[i][1] = ((Requirement) theList.get(i)).getReq();
			reqList[i][2] = ((Requirement) theList.get(i)).getRelNbr();
			if(complete) {
				reqList[i][3] = ((Requirement) theList.get(i)).getPriority1() + "";
				reqList[i][4] = ((Requirement) theList.get(i)).getPriority2() + "";
			}
		}
		return reqList;
	}
	
	/**
	 * Removes the specified requirement from the list
	 * 
	 * @param req a reference to the requirement to remove
	 */
	public void removeReq(Requirement req) {
	    theList.remove(req);
	}
	
	/**
	 * Returns a copy of the list where requirements that miss requirement 
	 * numbers are removed
	 * 
	 * @return the clamped copy
	 */
	public RequirementList getClampedCopy() {
		RequirementList list = new RequirementList();
		list.setPrioMethod(prioMethod);
		list.setPrioCriteria1(prioCriteria1);
		list.setPrioCriteria2(prioCriteria2);
		for(int i=0;i<getLength();i++) {
		    if (((Requirement) theList.get(i)).getReqNbr().length() > 0) {
		        list.addRequirement(((Requirement) theList.get(i)).getCopy());
		    }
		    
		}
		return list;
	}
	
	/**
	 * Returns the number of requirements in the list
	 * 
	 * @return the number of requirements in the list
	 */
	public int getLength() {
		return theList.size();
	}
		
	/**
	 * Returns the first prioritization criteria
	 * 
	 * @return the first prioritization criteria.
	 */
	public String getPrioCriteria1() {
		return prioCriteria1;
	}
	
	/**
	 * Sets the first prioritization criteria
	 *
	 * @param prioCriteria1 the first prioritization criteria to set.
	 */
	public void setPrioCriteria1(String prioCriteria1) {
		this.prioCriteria1 = prioCriteria1;
	}
	
	/**
	 * Returns the second prioritization criteria
	 * 
	 * @return the second prioritization criteria.
	 */
	public String getPrioCriteria2() {
		return prioCriteria2;
	}
	
	/**
	 * Sets the second prioritization criteria
	 *
	 * @param prioCriteria2 the second prioritization criteria to set.
	 */
	public void setPrioCriteria2(String prioCriteria2) {
		this.prioCriteria2 = prioCriteria2;
	}
	
	/**
	 * Returns the prioritization method used
	 * 
	 * @return the prioritization method.
	 */
	public int getPrioMethod() {
		return prioMethod;
	}
	
	/**
	 * Sets the prioritization method
	 * 
	 * @param prioMethod prioritization method to set.
	 */
	public void setPrioMethod(int prioMethod) {
		this.prioMethod = prioMethod;
	}
	
	/**
	 * Sorts and returns the list
	 * 
	 * @param k decides the sort criteria:<br>
	 *  k=0 -> Requirement number <br>
	 *  k=1 -> Requirement description <br>
	 *  k=2 -> Release number <br>
	 *  k=3 -> first prioritization criteria <br> 
	 *  k=4 -> second prioritization criteria
	 * @return the sorted RequirementList
	 */
	public RequirementList getSortedList(int k) {
	    /* 
	     * Check if the sorting shall be ascending or descending.
	     * If the same column is sorted twice with no other sorting in between
	     * the sorting order shall change
	     */
	    if(lastSortColumn==k) {
	        sortInvert = !sortInvert;
	    }
	    else {
	        sortInvert = false;
	    }
	    lastSortColumn = k;
	    
	    /* Other than doubles can be compared */
		if (k<3 || prioMethod != PLANNING_GAME) {
			for (int i=0; i<getLength()-1; i++) {
				for (int j=0; j<getLength()-1-i; j++) {
					String str1 = "";
					String str2 = "";
					int int1 = -1;
					int int2 = -1;
					double dob1 = -1.0;
					double dob2 = -1.0;
					switch (k) {
						case 0:
						    str1 = ((Requirement) theList.get(j+1)).getReqNbr();
							str2 = ((Requirement) theList.get(j)).getReqNbr();
							/* Check if req. # is int only */
						    try {
						        int1 = (new Integer(str1)).intValue();
						        int2 = (new Integer(str2)).intValue();
						    }
						    /* Req. # is not int only. */
						    catch (NumberFormatException nfe) {
						        /*
						         * Do nothing, Req. # is a String
						         */
						    }
							break;
						case 1:
							str1 = ((Requirement) theList.get(j+1)).getReq();
							str2 = ((Requirement) theList.get(j)).getReq();
							break;
						case 2:
							str1 = ((Requirement) theList.get(j+1)).getRelNbr();
							str2 = ((Requirement) theList.get(j)).getRelNbr();
							break;
						case 3:
						    dob1 = ((Requirement) theList.get(j+1)).getPriority1();
						    dob2 = ((Requirement) theList.get(j)).getPriority1();
						    break;
						case 4:
						    dob1 = ((Requirement) theList.get(j+1)).getPriority2();
						    dob2 = ((Requirement) theList.get(j)).getPriority2();
						    break;
					}
					/* If ints shall be compared */
					if(int1 != -1) {
					    if (((Requirement) theList.get(j+1)).getReqNbr().length() != 0 && 
					            aSmallerThanB(int1, int2) ) {  /* compare the two neighbors */
					        Requirement tmp = (Requirement) theList.remove(j);
					        theList.add(j+1, tmp);
					    }
					}
					/* If Strings shall be compared */
					else if(dob1 == -1.0) {
					    if (((Requirement) theList.get(j+1)).getReqNbr().length() != 0 && 
					            aSmallerThanB(str1,str2)) {  /* compare the two neighbors */
					        Requirement tmp = (Requirement) theList.remove(j);
					        theList.add(j+1, tmp);
					    }
					}
					/* If doubles shall be compared */
					else {
					    if (((Requirement) theList.get(j+1)).getReqNbr().length() != 0 && 
					           aSmallerThanB(dob1, dob2) ) {  /* compare the two neighbors */
					        Requirement tmp = (Requirement) theList.remove(j);
					        theList.add(j+1, tmp);
					    }
					}
				}
			}
		}
		
		/* Comparison between priorities set by Planning Game*/
		else {
			for (int i=0; i<getLength()-1; i++) {
				for (int j=0; j<getLength()-1-i; j++) {
					double d0 = 0.0;
					double d1 = 0.0;
					switch (k) {
						case 3:
							d0 = (new Double(((Requirement) theList.get(j)).getPriority1()).doubleValue());
							d1 = (new Double(((Requirement) theList.get(j+1)).getPriority1()).doubleValue());
							break;
						case 4:
							d0 = (new Double(((Requirement) theList.get(j)).getPriority2()).doubleValue());
							d1 = (new Double(((Requirement) theList.get(j+1)).getPriority2()).doubleValue());
							break;
					}
					/* 
					 * checks if: 
					 * 1: difference in pile belonging: ((d1*10)%10 < (d0*10)%10)
					 * 2: difference in order for reqs in same pile:
					 *    ((d1*10)%10 == (d0*10)%10) && (d1<d0))
					 */
					if (((Requirement) theList.get(j+1)).getReqNbr().length() != 0 && 
							( aSmallerThanB((d1*10)%10, (d0*10)%10) ||
									( ((d1*10)%10 == (d0*10)%10) && (aSmallerThanB(d1, d0))) ) ) {  /* compare the two neighbors */
						Requirement tmp = (Requirement) theList.remove(j);
						theList.add(j+1, tmp);
					}
				}
			}
		}
		return this;
		
		
	}
	
	/**
	 * Checks if a shall be considered smaller than b  depending on the sort order
	 * 
	 * @param a first int
	 * @param b second int
	 * @return true if a is smaller than b and normal sort is done
	 */
	private boolean aSmallerThanB(int a, int b) {
	    return (!sortInvert ? (a<b) : !(a<b)) ;
	}
	
	/**
	 * Checks if a shall be considered smaller than b  depending on the sort order
	 * 
	 * @param a first double
	 * @param b secod double
	 * @return true if a is smaller than b and normal sort is done
	 */
	private boolean aSmallerThanB(double a, double b) {
	    return (!sortInvert ? (a<b) : (b<a)) ;
	}
	
	/**
	 * Checks if a shall be considered smaller than b  depending on the sort order
	 * 
	 * @param a first String
	 * @param b second String
	 * @return true if a is smaller than b and normal sort is done
	 */
	private boolean aSmallerThanB(String a, String b) {
	    return (!sortInvert ? (a.compareTo(b)<0) : (b.compareTo(a)<0));
	}
}
