/*
 * ExcelWriter.java
 *
 * Created on 2005-feb-17
 */

package util;

import java.awt.Image;
import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import jxl.Workbook;
import jxl.write.Label;
import jxl.write.*;

/**
 * Defines the class used for saving pictures and information to an Excel file.
 * 
 * @author Mikael Jnsson & Per Klingns
 */
public class ExcelWriter {

    private WritableWorkbook workbook;  
    private WritableSheet tableSheet, graphSheet;
    private WritableFont reqXFont, headerFont, plainTextFont;
    private WritableCellFormat reqXFormat, headerFormat, plainTextFormat, reqHeaderFormat;
    
    /**
     * Creates an ExcelWriter, prepares the Excel workbook to save to
     * and sets the formatting.
     * 
     * @param file the file to save to
     */
    public ExcelWriter(File file) {
        try {
            workbook = Workbook.createWorkbook(file);
        } catch (IOException ioe) {
            JOptionPane.showMessageDialog(null, "Unable to create Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
        }
        /* Set the cell formatting */
        reqXFont = new WritableFont(WritableFont.ARIAL, 10, 
                WritableFont.BOLD, false);
        reqXFormat = new WritableCellFormat(reqXFont);
        headerFont = new WritableFont(WritableFont.ARIAL, 10, 
                WritableFont.BOLD, false);
        headerFormat = new WritableCellFormat(headerFont);
        plainTextFont = new WritableFont(WritableFont.ARIAL, 10, 
                WritableFont.NO_BOLD, false);
        plainTextFormat = new WritableCellFormat(plainTextFont);
        reqHeaderFormat = new WritableCellFormat(headerFont);
        try {
            reqXFormat.setAlignment(jxl.format.Alignment.CENTRE);
            reqXFormat.setVerticalAlignment(jxl.format.VerticalAlignment.CENTRE);
            plainTextFormat.setWrap(true);
            plainTextFormat.setVerticalAlignment(jxl.format.VerticalAlignment.TOP);
            reqHeaderFormat.setAlignment(jxl.format.Alignment.CENTRE);
        } 
        catch (WriteException we) {
            /* 
             * Do nothing. If exception is thrown, 
             * no special formats will be used 
             */
        }
    }
    
    /**
     * Writes the checked rows in the tables in root-cause window to  
     * the Excel file.
     * 
     * @param rootCauseTable the table containing root-causes and improvement 
     * suggestions (the left table)
     * @param requirementsTable the table where requirements are added as 
     * columns and linked to root-causes
     * @param sheetName the desired sheet name in the workbook
     */
    public void writeTable(JTable rootCauseTable, JTable requirementsTable, String sheetName) {
        tableSheet = workbook.createSheet(sheetName, 0);
        try {
            /* Add root-cause and improvement headers */
            tableSheet.setColumnView(0, 30);
            tableSheet.setColumnView(1, 30);
            tableSheet.addCell(new Label(0, 2, rootCauseTable.getColumnName(0), headerFormat));
            tableSheet.addCell(new Label(1, 2, rootCauseTable.getColumnName(1), headerFormat));
            
            /* Add requirement headers */
            for(int i=0;i<requirementsTable.getColumnCount();i++) {
                tableSheet.setColumnView(i+2, 10);
                tableSheet.addCell(new Label(i+2, 2, requirementsTable.getColumnName(i), 
                        reqHeaderFormat));
            }
            
            /* Add following rows */
            int nbrOfRowsToWrite = rootCauseTable.getRowCount();
            int rowInExcel = 3;
            for(int i=0;i<nbrOfRowsToWrite;i++) {
                /* If <print> is checked */
                if(((java.lang.Boolean) rootCauseTable.getValueAt(i,2))
                        .booleanValue() != false) {
                    for(int k=0;k<rootCauseTable.getColumnCount() + 
                    	requirementsTable.getColumnCount()-1;k++) { 
                        /* If root-cause or improvement column */
                        if(k < 2) {
                            if(rootCauseTable.getValueAt(i,k) != null) {
                                tableSheet.addCell(new Label(k, rowInExcel, 
                                        (String) rootCauseTable.getValueAt(i,k), 
                                        plainTextFormat));
                            }
                        }
                        /* If requirement column */
                        else {
                            if(requirementsTable.getValueAt(i,k-2) != null) {
                                Label l = new Label(k, rowInExcel, 
                                        (String) requirementsTable.getValueAt(i,k-2), 
                                        reqXFormat);
                                tableSheet.addCell(l); 
                            }
                        }
                    }
                    rowInExcel++;
                }       
            }
        }
        catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Unable to write table to Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        }  
    }
    
    /**
     * Saves the graph from GraphWindow to a file and to the Excel sheet.
     * 
     * @param image the image to be saved
     * @param sheetName the sheet name to save the graph in
     */
    public void writeGraph(Image image, String sheetName) {
        graphSheet = workbook.createSheet(sheetName, 1);
        try {
		    ImageIO.write((RenderedImage)image, "png", new File("graph.png"));
		    WritableImage wi = new WritableImage(0,0,9,30,new File("graph.png"));
		    graphSheet.addImage(wi);
        }
        catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Unable to write graph to Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /**
     * Flushes the write cache and closes the file.
     */
    public void writeAndCloseWorkbook() {
        try {
            workbook.write(); 
            workbook.close();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Error when writing to Excel file",
					"Failed", JOptionPane.ERROR_MESSAGE);
        }
    }  
}
